/* 
 * File:   mymodem.h
 * Author: david
 * Java style mixed declarations but ...
 *
 * Created on January 5, 2009, 11:07 AM
 */

#ifndef _MYMODEM_H
#define	_MYMODEM_H

static const int NBAUDS = 5;
static const unsigned int BAUDS[] = { B115200, B57600, B38400, B19200, B9600 };
    
class MyModem {
public:
    MyModem();
    ~MyModem() { }

    int open(  bool readonly );
    int openSession();
    int close();
    int test( const QString name, bool init = FALSE );
    int Init();
    int timed_read();
    void set_timeout(long ms_timeout);
    bool timed_out();
    int talk(const char* cmd, const char* resp, long timeout = MSEC_READ_TIMEOUT);
    bool testresp(const char* resp);
    int testdialout();
    int volume(int vol /* 0 .. 3 */);
    int dtmf(char tone /* 0 .. 9, A .. F, #, *, ! */);
    int dial(const char* phonenumber);
    char* bytes_read() { return buffer; }
    void lock();
    void unlock();
    int fn() { return m_fn; }
    bool justOpened() { return m_bJustOpened; }
    void setToOpen( bool toopen ) { m_bOpenNextTime = toopen; }
    void setFilename( QString name ) { m_modemFilename = name; m_bOpenNextTime = TRUE; }
    void setInitString( QString inits ) { m_modemInits = inits; m_bOpenNextTime = TRUE; }
    int connection() { return m_fn < 0 ? 0 :  NBAUDS - m_bno; }


private:
    int serial_setup( speed_t );
    int write(const char* cmd) { return ::write(m_fn, cmd, strlen(cmd)); }
    int UnlockReturn(int);

private:
    static const int MXCHAR = 64;
    char buffer[ MXCHAR ];
    struct timeval tv;
    int m_fn;
    bool m_bJustOpened;
    bool m_bOpenNextTime;
    bool m_bCIDOK;
    QString m_modemFilename;
    QString m_modemInits;
    long m_lDialTimeout;
    int m_bno;
};


// MODEM class members:
static QMutex mutex;

MyModem::MyModem() {
    m_fn = -1;
    m_bJustOpened = m_bOpenNextTime = FALSE;
    m_lDialTimeout = MSEC_DIAL_TIMEOUT; //default
    m_bno = 0;
}

int MyModem::serial_setup( speed_t baud )
{
    struct termios attr;

    if (tcgetattr(m_fn, &attr) < 0)
       return -1;

    attr.c_iflag = 0;
    attr.c_oflag = 0;
    attr.c_cflag = CLOCAL | CREAD | CS8;
    attr.c_lflag = 0;
    attr.c_cc[VTIME] = 0;       /* timeout in tenths of a second */
    attr.c_cc[VMIN] = 1;        /* Only wait for a single char */

    cfsetispeed(&attr, baud);
    cfsetospeed(&attr, baud);

    return tcsetattr(m_fn, TCSAFLUSH, &attr);
}

int MyModem::openSession() {
    m_bJustOpened = FALSE;

    if (m_fn >= 0) {
        if (m_bOpenNextTime)
            close();
        else
            return m_fn;
    }

    if (open(FALSE) < 0)
        return -1;

    // DISABLE ECHO ALWAYS
    struct termios tios;

    ioctl(m_fn, TCGETS, &tios);
    tios.c_lflag &= ~(ECHO); /*|ECHOE|ECHOK|ECHOKE|ECHOCTL*/
    ioctl(m_fn, TCSETS, &tios);

    return m_fn;
}

int MyModem::open( bool readonly ) {
    int options;

    if (readonly ) {
        options = O_NOCTTY | O_RDONLY | O_NDELAY ;
    }
    else {
        options = O_NOCTTY | O_RDWR | O_NDELAY ;
    }

    if  (( m_fn = ::open(m_modemFilename.toAscii(), options)) < 0) {
        return -1;
    }

    if  ( m_fn >= 0 )
        m_bJustOpened = TRUE;

    return m_fn;
}

int MyModem::close() {
    if  ( m_fn < 0 )
        return 0;

    int ret =  ::close(m_fn);
    m_fn = -1;

    return ret;
}

int MyModem::test( const QString name, bool init ) {
    int flags = F_OK | R_OK;
    //if  ( !readonly )
        flags |= W_OK;
    if  ( access( name.toAscii(), flags ) != 0 )
        return -1;

    if  (!init)
        return 0;
    
    // get reliable connect speed.
    setFilename( name );
    if  ( openSession() < 0 )
        return -1;
    
    // try to initialize, find buad rate that will work as sometimes things are borked
    m_bno = 0;
    while (talk(m_modemInits.toAscii(), MODEM_WAS_OK /*, MSEC_RESET_TIMEOUT */) != 0) {
        if (++m_bno >= NBAUDS)
            return ERRNO_INITMODEM;
        serial_setup(BAUDS[ m_bno ]);
    }

    // test some other parameters that are not subject to change (hardware, subscription)
    QString cid = MODEM_CID;
    cid += "?"; // try to get current parameter if any
    m_bCIDOK = !talk( cid.toAscii(), MODEM_WAS_ERROR );     // :-)

    // done, close it now!
    return close();
}

// simple convenience functions
void MyModem::lock() {
    mutex.lock();
}

void MyModem::unlock() {
    mutex.unlock();
}

int MyModem::UnlockReturn( int ret ) {
    unlock();
    return ret;
}

int MyModem::Init() {
    if  ( !m_bJustOpened )
        return 0;

    return talk( m_modemInits.toAscii(), MODEM_WAS_OK /*, MSEC_RESET_TIMEOUT */);
}

bool MyModem::testresp( const char* resp ) {
    if  (!resp )
        return TRUE;   // not test OK
    if  (!*resp )
        return TRUE;

    return strcasestr( buffer, resp ) != NULL;
}

int MyModem::testdialout() {
        if  (testresp( MODEM_BUSY))
            return ERRNO_BUSY;
        if  (testresp( MODEM_CARRIER ))
            return ERRNO_NOCARRIER;
        if  (testresp( MODEM_DIALTONE ))
            return ERRNO_NODIALTONE;
        if  (testresp( MODEM_NOANSWER ))
            return ERRNO_NOANSWER;
        if  (testresp( MODEM_WAS_ERROR ))
            return ERRNO_GENERAL;
        return 0;
}
/*
 * Tne next two function were written by Nelson Castillo.
 * You should not blame Dave for what he wrote here :-)
 */

int MyModem::timed_read() {
    fd_set rfds;
    int retval;
    int nread = 0;

    FD_ZERO(&rfds);
    FD_SET(m_fn, &rfds);

select_next:

    /* We rely on the value of tv after the select call.
     * This is a Linux-only thing.
     * TODO: Fix. */

    errno = 0;
    retval = select(m_fn + 1, &rfds, NULL, NULL, &tv);

    if (retval < 0) {
        if (errno == EINTR)
            goto select_next;

        return -1;
    }
    else if (retval) {
        if (MXCHAR - nread > 0) {
            int r;
            r = ::read(m_fn, buffer + nread, MXCHAR - nread);
            if (r > 0) {
                nread += r;

                if (MXCHAR - nread > 0)
                    goto select_next;
            }
        }
    }

    return nread;
}

void MyModem::set_timeout(long ms_timeout) {
    tv.tv_sec = ms_timeout / 1000;
    tv.tv_usec = (ms_timeout % 1000) * 1000;
};

bool MyModem::timed_out() {
    return tv.tv_sec <= 0 && tv.tv_usec <= 0;
};

int MyModem::talk( const char* cmd, const char* resp, long timeout ) {
    int ret;

    if  ( m_fn < 0 )
        return -1;

    lock();

    tcflush( m_fn, TCIOFLUSH );

    for (int t = 0; t < RETRIES; ++t) {
        ret = write( cmd );

        if ( ret < 0 )
            break;
        write( "\r" );       // general terminator ^M

        if (ret != strlen(cmd))
            continue;

        set_timeout( timeout );
        timed_read();

        if (testresp( resp ) )
            return UnlockReturn( 0 );
        // need valid test
    }

    return UnlockReturn( ERRNO_GENERAL );
}

int MyModem::dial(const char *cmd) {
    int ret;
    
    if  ( m_fn < 0 )
        return -1;

    m_lDialTimeout = MSEC_DIAL_TIMEOUT; // default

    if (talk(MODEM_READ_S6, 0) == 0) {
        QString numb = buffer;
        int res = numb.toInt();
        if (res > 0)
            m_lDialTimeout = res * 1000l + MSEC_DIAL_ADDTIMEOUT;
    }

    lock();

    tcflush( m_fn, TCIOFLUSH );

    ret = write( cmd );

    if (ret < 0)
        return UnlockReturn( ERRNO_DIAL );

    write( "\r" ); // general terminator ^M

    if (ret != strlen(cmd))
        return UnlockReturn( ERRNO_DIAL );

    set_timeout( m_lDialTimeout );
    do {
        ret =timed_read();

        if (ret < 0)
            return UnlockReturn( ERRNO_DIAL );

        // return any specific responses
        ret = testdialout();
        if  ( ret < 0 )
            return UnlockReturn( ret );
        if (testresp( MODEM_WAS_OK ) )
            return UnlockReturn( 0 );
        if (testresp( MODEM_VOICE_COM ) )
            return UnlockReturn( 0 );
    }
    while (!timed_out());

    return UnlockReturn( 0 );
}

int MyModem::volume( int vol ) {
    switch (vol) {
        case 0: return talk( MODEM_ZERO_VOLUME, MODEM_WAS_OK );
        case 1: return talk( MODEM_LOW_VOLUME, MODEM_WAS_OK );
        case 2: return talk( MODEM_MED_VOLUME, MODEM_WAS_OK );
        case 3: return talk( MODEM_HI_VOLUME, MODEM_WAS_OK );
        default: return 0;
    }
}

int MyModem::dtmf( char tone ) {
    QString dtmf = MODEM_TONE;
    dtmf += tone;
    return talk( dtmf.toAscii(), MODEM_WAS_OK, MSEC_DTMF_TIMEOUT );
}



#endif	/* _MYMODEM_H */

