/* phoneapplet.cpp */

#include <QPainter>
#include <QFontMetrics>
#include <QSize>
#include <QtCore>
#include <KIcon>

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/package.h>

// includes for linux std c++, ioctl, etc
#include <errno.h>
#include <stdio.h>
#include <sys/file.h>
#include <stdlib.h>
#include <unistd.h>
#include <paths.h>
#include <fcntl.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <signal.h>
#include <termios.h>
#include <getopt.h>
#include <syslog.h>
#include <pwd.h>
#include <ctype.h>
#include <linux/version.h>
#include <linux/serial.h>
#include <printf.h>

#include <kiconloader.h>
#include <klineedit.h>
#include <kfiledialog.h>
#include <kconfigdialog.h>

#include "phoneapplet.h"
#include "ui_phonecall.h"
#include "ui_appletsetup.h"
#include "ui_appletsetupquick.h"
#include "ui_appletsetupfax.h"

#include "mymodem.h"
#include "phoneapplet_threads.h"
#include "phoneapplet_dialogs.h"
#include "phoneapplet_contacts.h"

PhoneApplet::PhoneApplet(QObject *parent, const QVariantList &args): Plasma::Applet(parent, args) {
    m_eState = eIDLE;
    m_iconMain = KIcon("internet-telephony");
    m_iconRing = KIcon( "appointment-reminder");
    m_iconRing2 = m_iconMain;
    m_iconConnect = KIcon( "media-seek-forward");
    m_iconConnect2 = m_iconMain;
    m_iconDial = KIcon("preferences-desktop-user");
    m_iconDial2 = m_iconMain;
    m_iconTalk = KIcon("user-identity");
    m_iconTalk2 = KIcon( "list-remove-user");
    m_iconHangup = m_iconTalk2;
    m_iconHangup2 = m_iconMain;
    m_iconNogood = KIcon( "user-busy" );
    m_iconNogood2 = KIcon( "preferences-system" );
    m_iconSendFax = m_iconConnect;
    m_iconSendFax2 = KIcon( "document-print-preview" );
    m_iconRecvFax = KIcon( "media-seek-backward");
    m_iconRecvFax2 = m_iconSendFax2;

    m_modem = new MyModem;
    m_QTimerBlink = new QTimer(this);
    m_QTimerEndRings = new QTimer(this);
    m_QTimerSeconds = new QTimer(this);
    m_QTimerMouse = new QTimer(this);
    m_DaemonThread = new RingDaemonThread(this);
    m_AnswerThread = new ModemAnswerThread(this);
    m_DialThread = new ModemDialThread(this);
    m_HangupThread = new ModemHangupThread(this);
    m_PostConnectThread = new PostConnectThread(this);
    m_SendFaxThread = new SendFaxThread(this);
    m_RecvFaxThread = new RecvFaxThread(this);
    m_InfoBox = new InfoBoxes(this);
    m_callsMade = new KHistoryComboBox(FALSE, 0);
    m_callsList = new MyListWidget();
    m_contactsList = new ContactsListWidget();
    m_PhoneNumber = new MyPhoneNumber(this);

    setBackgroundHints(DefaultBackground);
    resize(48, 48);
}

PhoneApplet::~PhoneApplet() {
    if (hasFailedToLaunch()) {
        // Do something
    } else {
        // Save settingsome cleanup here
        AbortAllThreads();

        delete m_modem;
        delete m_QTimerBlink;
        delete m_QTimerEndRings;
        delete m_QTimerSeconds;
        delete m_QTimerMouse;
        delete m_QTimerContacts;
        delete m_DaemonThread;
        delete m_AnswerThread;
        delete m_HangupThread;
        delete m_DialThread;
        delete m_PostConnectThread;
        delete m_SendFaxThread;
        delete m_RecvFaxThread;
        delete m_PhoneNumber;
        //delete m_AppletSetup;
        delete m_InfoBox;
        delete m_callsMade;
        delete m_callsList;
        delete m_contactsList;
    }
}

void PhoneApplet::AbortAllThreads() {
    if  ( m_modem->fn() >= 0 ) {
        m_HangupThread->setForceReset( TRUE );
        m_HangupThread->start();
        m_HangupThread->wait();
    }

    // try em all to make this close and end!
    m_PhoneNumber->setAttribute( Qt::WA_DeleteOnClose );
    m_PhoneNumber->setVisible(FALSE);
    m_PhoneNumber->close();
    m_QTimerBlink->stop();
    m_QTimerEndRings->stop();
    m_QTimerSeconds->stop();
    m_AnswerThread->terminate();
    m_HangupThread->terminate();
    m_DialThread->terminate();
    m_PostConnectThread->terminate();
    m_SendFaxThread->terminate();
    m_RecvFaxThread->terminate();

    AbortDaemon();
}

void PhoneApplet::init() {
    setAcceptDrops(TRUE);
    setHasConfigurationInterface(TRUE);

    propConfig = config(CONFIG_GROUP_PROPERTIES);
    optConfig = config(CONFIG_GROUP_OPTIONS);
    callConfig = config(CONFIG_GROUP_CALLS);

    // set this object before readConfig!
    m_callsMade->setVisible(FALSE); // not using a GUI!
    m_callsMade->setMaxCount(NQUICK);
    m_callsMade->setDuplicatesEnabled(FALSE); // NO duplicates
    //m_callsMade->setEditable(FALSE);        // NO linedit box

    readConfig();

    m_eState = eIDLE; // decent startup state, but must be reset to use
    m_bAlternate = FALSE;
    m_bMouseOver = FALSE;
    //m_bMousePress = FALSE;
    m_bUseSearch = FALSE;

    // set up timer
    m_QTimerBlink->setInterval(MSEC_BLINK);
    m_QTimerBlink->setSingleShot(FALSE);
    connect(m_QTimerBlink, SIGNAL(timeout()), this, SLOT(blinkit()));

    m_QTimerEndRings->setInterval(MSEC_AFTER_RINGS);
    m_QTimerEndRings->setSingleShot(TRUE);
    connect(m_QTimerEndRings, SIGNAL(timeout()), this, SLOT(proc_endringseq()));

    m_QTimerSeconds->setInterval(MSEC_ONE_SECOND);
    m_QTimerSeconds->setSingleShot(FALSE);
    connect(m_QTimerSeconds, SIGNAL(timeout()), this, SLOT(timeElapsed()));

    m_QTimerMouse->setInterval(MSEC_MOUSE_LIMIT);
    m_QTimerMouse->setSingleShot(TRUE);
    //connect( m_QTimerMouse, SIGNAL(timeout()), this, SLOT( unclick()));

    // Connect up modem UI threads
    connect(m_AnswerThread, SIGNAL(finished()), this, SLOT(CallAnswered()));
    connect(m_DialThread, SIGNAL(finished()), this, SLOT(CallAnswered()));
    connect(m_HangupThread, SIGNAL(finished()), this, SLOT(CallHungUp()));
    connect(m_PostConnectThread, SIGNAL(finished()), this, SLOT(PostConnectEnd()));
    connect(m_SendFaxThread, SIGNAL(finished()), this, SLOT(faxSendDone()));
    connect(m_RecvFaxThread, SIGNAL(finished()), this, SLOT(faxRecvDone()));

    /*  set up the UI here as some of it did not work in the class itself
     *
     */
    m_InfoBox->setPhoneNumberDlg( m_PhoneNumber );
    m_PhoneNumber->UI.toolButton_Bell->setChecked(m_bWantSound);
    setVolumeIcon();
    setBellIcon();
    connect(m_PhoneNumber->UI.pushButton_Call, SIGNAL(clicked()), this, SLOT(UIButtonClick()));
    connect(m_PhoneNumber->UI.toolButton_Bell, SIGNAL(clicked()), this, SLOT(toggle_sound()));
    connect(m_PhoneNumber->UI.toolButton_Volume, SIGNAL(clicked()), this, SLOT(clickVolume()));
    connect(m_PhoneNumber->UI.toolButton_Info, SIGNAL(clicked()), this, SLOT(InfoBox()));
    connect(m_PhoneNumber->UI.toolButton_Config, SIGNAL(clicked()), this, SLOT(showConfigurationInterface()));
    connect(m_PhoneNumber->UI.pushButton_Back, SIGNAL(clicked()), this, SLOT(deleteText()));
    connect(m_PhoneNumber->UI.pushButton_1, SIGNAL(clicked()), this, SLOT(digitClicked1()));
    connect(m_PhoneNumber->UI.pushButton_2, SIGNAL(clicked()), this, SLOT(digitClicked2()));
    connect(m_PhoneNumber->UI.pushButton_3, SIGNAL(clicked()), this, SLOT(digitClicked3()));
    connect(m_PhoneNumber->UI.pushButton_4, SIGNAL(clicked()), this, SLOT(digitClicked4()));
    connect(m_PhoneNumber->UI.pushButton_5, SIGNAL(clicked()), this, SLOT(digitClicked5()));
    connect(m_PhoneNumber->UI.pushButton_6, SIGNAL(clicked()), this, SLOT(digitClicked6()));
    connect(m_PhoneNumber->UI.pushButton_7, SIGNAL(clicked()), this, SLOT(digitClicked7()));
    connect(m_PhoneNumber->UI.pushButton_8, SIGNAL(clicked()), this, SLOT(digitClicked8()));
    connect(m_PhoneNumber->UI.pushButton_9, SIGNAL(clicked()), this, SLOT(digitClicked9()));
    connect(m_PhoneNumber->UI.pushButton_0, SIGNAL(clicked()), this, SLOT(digitClicked0()));
    connect(m_PhoneNumber->UI.pushButton_Asterisk, SIGNAL(clicked()), this, SLOT(digitClickedAst()));
    connect(m_PhoneNumber->UI.pushButton_Pound, SIGNAL(clicked()), this, SLOT(digitClickedPnd()));
    connect(m_PhoneNumber->UI.toolButton_Flash, SIGNAL(clicked()), this, SLOT(digitClickedFlsh()));
    connect(m_PhoneNumber->UI.toolButton_Redial, SIGNAL(clicked()), this, SLOT(RedialCall()));
    connect(m_PhoneNumber->UI.toolButton_Up, SIGNAL(clicked()), this, SLOT(clickPrev()));
    connect(m_PhoneNumber->UI.toolButton_Down, SIGNAL(clicked()), this, SLOT(clickNext()));
    connect(m_PhoneNumber->UI.toolButton_Contacts, SIGNAL(clicked()), this, SLOT(startContacts()));
    connect(m_PhoneNumber->UI.toolButton_Fax, SIGNAL(clicked()), this, SLOT(doFax()));
    connect(m_PhoneNumber->UI.lineEdit_Number, SIGNAL(textEdited(QString)), this, SLOT(doTextEdited(QString)));
    connect(m_PhoneNumber->UI.lineEdit_Number, SIGNAL(textChanged(QString)), this, SLOT(doTextChanged(QString)));
    // for diagnostic, maybe useful purpose?
    //connect(m_PhoneNumber->UI.toolButton_Status4, SIGNAL(clicked()), this, SLOT(toggle_daemon()));

    m_InfoBox->setModal(FALSE);
    m_InfoBox->setVisible(FALSE);
    m_InfoBox->addButton(QMessageBox::Close);
    m_InfoBox->setPalette(QPalette(Qt::white));
    m_InfoBox->setWindowTitle(name());

    // connect the history, only does anything if shown, of course
    connect(m_callsMade, SIGNAL(highlighted(int)), this, SLOT(historySelect(int)));

    //addAction(ACTION_CONFIG, new QAction(KIcon("preferences-system"), "Configure PhoneApplet", this));
    addAction(ACTION_ABOUT, new QAction(KIcon("help-about"), "About PhoneApplet", this));
    //addAction(ACTION_GUI, new QAction(KIcon("phone"), "Show/hide Phone Dialog", this));

    //connect(action(ACTION_CONFIG), SIGNAL(triggered()), this, SLOT(configDialog()));
    connect(action(ACTION_ABOUT), SIGNAL(triggered()), this, SLOT(InfoBox()));
    //connect(action(ACTION_GUI), SIGNAL(triggered()), this, SLOT(ToggleGUI()));

    connect(m_DaemonThread, SIGNAL(started()), this, SLOT(setDaemonIcon()));
    connect(m_DaemonThread, SIGNAL(finished()), this, SLOT(setDaemonIcon()));

    // do this stuff here to TEST modem access!
    if  ( m_modem->test( m_strModemFilename, TRUE ) < 0) {
        ErrMessage( ERRNO_NOMODEM );
        set_state( eNOGOOD );
        return;
    }

    StartDaemon();

    connect(m_contactsList, SIGNAL(currentRowChanged(int)), this, SLOT(contactsSelect(int)));
    connect(m_contactsList, SIGNAL(clicked(const QModelIndex&)), this,
                                SLOT(contactsClick(const QModelIndex&)));

    // Contacts load in a non-friendly, asynchronous manner so will play...
    // The dataengine does not cooperate with explicit threads
    // Timers are not explicit threads, apparently, so ...
    m_QTimerContacts = new QTimer(this);
    m_QTimerContacts->setInterval(MSEC_ONE_SECOND);
    m_QTimerContacts->setSingleShot(FALSE);
    connect(m_QTimerContacts, SIGNAL(timeout()), this, SLOT(contactsOps()));
    m_eContactsData = eNONE;

    if  ( m_bWantEngine )
        startEngine();
    if  ( m_bMergeQuick )
        addQuickContacts();

    connect( this, SIGNAL(activate()), this, SLOT(ToggleGUI()));
}

void PhoneApplet::startEngine() {
    m_PhoneNumber->UI.toolButton_Contacts->setEnabled( FALSE );
    m_Contacts = dataEngine( CONTACT_ENGINE );   // should never produce error
    m_QTimerContacts->start();
}

QList< QAction * > PhoneApplet::contextualActions() {
    static QList< QAction* > actions;
    //actions << action( ACTION_GUI );
    //actions << action( ACTION_CONFIG );
    actions << action( ACTION_ABOUT );
    return actions;
}

void PhoneApplet::newAppletSetup() {
    m_AppletSetup = new MyAppletSetup();

    m_InfoBox->setAppletSetupDlg( m_AppletSetup );
    connect(m_AppletSetup, SIGNAL(accepted()), this, SLOT(saveConfigDialog()));
    connect(m_AppletSetup->UI.toolButton_Modem, SIGNAL(clicked()), this, SLOT(modemFileDialog()));
    connect(m_AppletSetup->UI.toolButton_Player, SIGNAL(clicked()), this, SLOT(test_ring()));
    connect(m_AppletSetup->UI.toolButton_Sound, SIGNAL(clicked()), this, SLOT(soundFileDialog()));
    connect(m_AppletSetup->UI.toolButton_Init, SIGNAL(clicked()), this, SLOT(modemInitsBox()));
    connect(m_AppletSetup->UI.toolButton_ModemReset, SIGNAL(clicked()), this, SLOT(modemResetBox()));
    connect(m_AppletSetup->UI.toolButton_Contacts, SIGNAL(clicked()), this, SLOT(contactsBox()));

    m_FaxSetup = new MyFaxSetup();

    m_InfoBox->setFaxSetupDlg( m_FaxSetup );
    connect(m_FaxSetup->UI.toolButton_SendFax, SIGNAL(clicked()), this, SLOT( sendFaxCmdBox()));
    connect(m_FaxSetup->UI.toolButton_Convert, SIGNAL(clicked()), this, SLOT( convertToFaxCmdBox()));
    connect(m_FaxSetup->UI.toolButton_RecvFax, SIGNAL(clicked()), this, SLOT( recvDialFaxCmdBox()));
    connect(m_FaxSetup->UI.toolButton_PickupFax, SIGNAL(clicked()), this, SLOT( recvPickupFaxCmdBox()));
    connect(m_FaxSetup->UI.toolButton_Prefix, SIGNAL(clicked()), this, SLOT( faxPrefixBox()));
    connect(m_FaxSetup->UI.toolButton_ViewFax, SIGNAL(clicked()), this, SLOT( faxViewerBox()));
}

void PhoneApplet::newQuickListSetup() {
    m_QuickSetup = new QTabWidget();

    int nquick = m_quickDial.count();
    int npanes = nquick / NQUICK + 1;

    m_QuickPanes.clear();
    for ( int p = 0; p < npanes; ++p ) {
        m_QuickPanes << new MyQuickSetup();
        QString label;
        if  ( p < 1 )
            QTextStream( &label ) << '#';
        QTextStream( &label ) << p * NQUICK << '-'  << ( p + 1 ) * NQUICK - 1;
        m_QuickSetup->addTab( m_QuickPanes[ p ], label );
    }

    m_InfoBox->setQuickSetupDlg( m_QuickSetup );

    m_quickEdit.clear();
    m_quickButton.clear();

    for (int q = 0; q < npanes * NQUICK; ++q) {
        QString label;
        if  ( q < NQUICK )
            QTextStream( &label ) << "#";
        QTextStream( &label ) << q;

        int p = q / NQUICK;
        m_quickEdit << new QLineEdit( m_QuickPanes[ p ] );
        m_quickButton << new QToolButton( m_QuickPanes[ p ] );
        m_quickButton[ q ]->setText(label);
        m_quickButton[ q ]->setPalette(m_AppletSetup->UI.toolButton_Modem->palette());
        m_QuickPanes[ p ]->UI.formLayout_Quick->addRow(m_quickButton[ q ], m_quickEdit[ q ]);
    }
    
    for (int p = 0; p < npanes; ++p) {
        int np = p * NQUICK;
        connect(m_quickButton[ np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked0()));
        connect(m_quickButton[ 1 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked1()));
        connect(m_quickButton[ 2 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked2()));
        connect(m_quickButton[ 3 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked3()));
        connect(m_quickButton[ 4 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked4()));
        connect(m_quickButton[ 5 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked5()));
        connect(m_quickButton[ 6 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked6()));
        connect(m_quickButton[ 7 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked7()));
        connect(m_quickButton[ 8 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked8()));
        connect(m_quickButton[ 9 + np ], SIGNAL(clicked()), this, SLOT(quickPaneClicked9()));
    }
}

void PhoneApplet::readConfig() {
    m_strModemFilename = propConfig.readEntry( CONFIG_MODEM_FILENAME, MODEM_FILENAME );
    m_strNewModemFilename = m_strModemFilename;
    m_modem->setFilename( m_strModemFilename );
    m_strModemInits = propConfig.readEntry( CONFIG_MODEM_INITS, MODEM_INITS );
    m_modem->setInitString( m_strModemInits );
    m_strPlaySound = propConfig.readEntry( CONFIG_PLAY_SOUND, CMD_PLAY_SOUND );
    m_strSoundFile = propConfig.readEntry( CONFIG_SOUND_FILE, PLAY_SOUND_FILE );
    m_bWantReset = propConfig.readEntry( CONFIG_WANT_RESET, true );
    m_bWantKeep = propConfig.readEntry( CONFIG_WANT_KEEP, false );
    m_strFaxSendCmd = propConfig.readEntry( CONFIG_FAX_SEND, CMD_FAX_SEND );
    m_strFaxRecvCmd = propConfig.readEntry( CONFIG_FAX_RECV, CMD_FAX_RECV );
    m_strFaxPickupCmd = propConfig.readEntry( CONFIG_FAX_PICKUP, CMD_FAX_PICKUP );
    m_strFileToFaxCmd = propConfig.readEntry( CONFIG_FILE2FAX, CMD_FILE2FAX );
    m_strFaxViewCmd = propConfig.readEntry( CONFIG_FAX_VIEWER, CMD_FAX_VIEWER );
    m_strFaxPrefix = propConfig.readEntry( CONFIG_FAX_PREFIX, CMD_FAX_PREFIX );
    m_bWantFaxOutput = propConfig.readEntry( CONFIG_FAX_OUTBOX, TRUE );

    
    m_bWantSound = optConfig.readEntry( CONFIG_WANT_SOUND, true );
    m_iVolume = optConfig.readEntry( CONFIG_MODEM_VOLUME, 2 );
    m_bVolumeRising = TRUE; // next one is 3
    m_bWantSearch = optConfig.readEntry( CONFIG_WANT_SEARCH, true );
    m_bMergeQuick = optConfig.readEntry( CONFIG_SEARCH_QUICK, true );
    m_bWantEngine = optConfig.readEntry( CONFIG_USE_ENGINE, false );

    m_quickDial.clear();
    for ( int q = 0; q < NQUICK; ++q )
        m_quickDial << "";
    m_quickDial = callConfig.readEntry( CONFIG_QUICKDIAL, m_quickDial );

    m_callsMade->clearHistory();
    m_callsMade->setHistoryItems( callConfig.readEntry( CONFIG_CALLSMADE, m_callsMade->historyItems() ), FALSE );
    m_iInHistory = 0;

    m_iInContacts = 0;
}

void PhoneApplet::saveConfig() {
    propConfig.writeEntry( CONFIG_MODEM_FILENAME, m_strNewModemFilename );
    propConfig.writeEntry( CONFIG_MODEM_INITS, m_strModemInits );
    propConfig.writeEntry( CONFIG_PLAY_SOUND, m_strPlaySound );
    propConfig.writeEntry( CONFIG_SOUND_FILE, m_strSoundFile );
    propConfig.writeEntry( CONFIG_WANT_RESET, m_bWantReset );
    propConfig.writeEntry( CONFIG_WANT_KEEP, m_bWantKeep );
    propConfig.writeEntry( CONFIG_FAX_SEND, m_strFaxSendCmd );
    propConfig.writeEntry( CONFIG_FAX_RECV,  m_strFaxRecvCmd );
    propConfig.writeEntry( CONFIG_FAX_PICKUP, m_strFaxPickupCmd );
    propConfig.writeEntry( CONFIG_FAX_OUTBOX, m_bWantFaxOutput );
    propConfig.writeEntry( CONFIG_FILE2FAX, m_strFileToFaxCmd );
    propConfig.writeEntry( CONFIG_FAX_VIEWER, m_strFaxViewCmd );
    propConfig.writeEntry( CONFIG_FAX_PREFIX, m_strFaxPrefix );
    
    optConfig.writeEntry( CONFIG_WANT_SOUND, m_bWantSound );
    optConfig.writeEntry( CONFIG_MODEM_VOLUME, m_iVolume );
    optConfig.writeEntry( CONFIG_WANT_SEARCH, m_bWantSearch );
    optConfig.writeEntry( CONFIG_SEARCH_QUICK, m_bMergeQuick );
    optConfig.writeEntry( CONFIG_USE_ENGINE, m_bWantEngine );

    callConfig.writeEntry( CONFIG_QUICKDIAL, m_quickDial );
    callConfig.writeEntry( CONFIG_CALLSMADE, m_callsMade->historyItems());
}

void PhoneApplet::resetConfigDialog() {
    m_bConfigChanged = FALSE;
    
    m_AppletSetup->UI.lineEdit_Modem->setText( m_strModemFilename );
    m_AppletSetup->UI.lineEdit_Init->setText( m_strModemInits );
    m_AppletSetup->UI.lineEdit_Init->setCursor(0);  // this is long
    m_AppletSetup->UI.lineEdit_Player->setText( m_strPlaySound );
    m_AppletSetup->UI.lineEdit_Sound->setText( m_strSoundFile );
    m_AppletSetup->UI.checkBox_ModemReset->setChecked( m_bWantReset );
    m_AppletSetup->UI.checkBox_ModemKeep->setChecked( m_bWantKeep );
    m_AppletSetup->UI.checkBox_ContactsSearch->setChecked( m_bWantSearch );
    m_AppletSetup->UI.checkBox_SearchQuick->setChecked( m_bMergeQuick );
    m_AppletSetup->UI.checkBox_Akonadi->setChecked( m_bWantEngine );
    
    m_FaxSetup->UI.lineEdit_SendFax->setText( m_strFaxSendCmd );
    m_FaxSetup->UI.lineEdit_Convert->setText( m_strFileToFaxCmd );
    m_FaxSetup->UI.lineEdit_RecvFax->setText( m_strFaxRecvCmd );
    m_FaxSetup->UI.lineEdit_PickupFax->setText( m_strFaxPickupCmd );
    m_FaxSetup->UI.lineEdit_ViewFax->setText( m_strFaxViewCmd );
    m_FaxSetup->UI.lineEdit_Prefix->setText( m_strFaxPrefix );
    m_FaxSetup->UI.checkBox_ShowOutput->setChecked( m_bWantFaxOutput );

    int  q;
    for ( q = 0; q < m_quickDial.count(); ++q )
        m_quickEdit[ q ]->setText( m_quickDial[ q ] );
    for ( ; q < m_quickEdit.count(); ++q )
        m_quickEdit[ q ]->clear();
}

void PhoneApplet::saveConfigDialog() {
    // cannot do this now!!
    m_strNewModemFilename = m_AppletSetup->UI.lineEdit_Modem->text();
    if (m_strNewModemFilename != m_strModemFilename) {
        if (m_DaemonThread->isRunning()) {
            if (m_InfoBox->wantNewModemBox()) {
                if (m_modem->test(m_strNewModemFilename, FALSE)) {
                    ErrMessage(ERRNO_NOMODEM);
                    m_strNewModemFilename = m_strModemFilename;
                }

            }
        }
        else {
            if (m_modem->test(m_strNewModemFilename, FALSE)) {
                ErrMessage(ERRNO_NOMODEM);
                m_strNewModemFilename = m_strModemFilename;
            }
            else {
                m_strModemFilename = m_strNewModemFilename;
                StartDaemon();
            }
        }
    }

    m_strPlaySound = m_AppletSetup->UI.lineEdit_Player->text();
    m_strSoundFile = m_AppletSetup->UI.lineEdit_Sound->text();
    m_bWantReset = m_AppletSetup->UI.checkBox_ModemReset->isChecked();
    m_bWantKeep = m_AppletSetup->UI.checkBox_ModemKeep->isChecked();
    m_modem->setToOpen( !m_bWantKeep );
    m_bWantSearch = m_AppletSetup->UI.checkBox_ContactsSearch->isChecked();
    m_bMergeQuick = m_AppletSetup->UI.checkBox_SearchQuick->isChecked();
    bool bHadEngine = m_bWantEngine;
    m_bWantEngine = m_AppletSetup->UI.checkBox_Akonadi->isChecked();
    if  ( !bHadEngine && m_bWantEngine )
        startEngine();

    m_strFaxSendCmd = m_FaxSetup->UI.lineEdit_SendFax->text();
    m_strFaxRecvCmd = m_FaxSetup->UI.lineEdit_RecvFax->text();
    m_strFaxPickupCmd = m_FaxSetup->UI.lineEdit_PickupFax->text();
    m_strFaxViewCmd = m_FaxSetup->UI.lineEdit_ViewFax->text();
    m_strFaxPrefix = m_FaxSetup->UI.lineEdit_Prefix->text();
    m_strFileToFaxCmd = m_FaxSetup->UI.lineEdit_Convert->text();
    m_bWantFaxOutput = m_FaxSetup->UI.checkBox_ShowOutput->isChecked();

    QString strNewModemInits = m_AppletSetup->UI.lineEdit_Init->text();
    if  ( strNewModemInits != m_strModemInits ) {
        if  ( m_InfoBox->initAreYouSureBox() ) {
            m_strModemInits = strNewModemInits;
            m_modem->setInitString( m_strModemInits );
        }
    }

    // quick numbers
    removeQuickContacts();   // if any there, might have been changed :-)
    int nq = m_quickEdit.count();
    while ( nq > NQUICK && m_quickEdit[ nq - 1 ]->text().length() < 1 )
        --nq;
    m_quickDial.clear();
    for ( int q = 0; q < nq; ++q ) {
        m_quickDial << m_quickEdit[ q ]->text();
    }
    if  ( m_bMergeQuick )
        addQuickContacts();

    saveConfig();
    endConfigDialog();
}

void PhoneApplet::endConfigDialog() {
    m_InfoBox->setVisible( FALSE );
}

void PhoneApplet::soundFileDialog() {
    QString path = getenv( "KDEDIR"  );
    path += "/share/sounds";
    QString filter = "*.ogg *.mp3 *.wav|Sound files\n*|All files";
    QString filename = KFileDialog::getOpenFileName( KUrl( path ),
            filter, m_AppletSetup, name() );
    if  ( filename != "" )
        m_AppletSetup->UI.lineEdit_Sound->setText( filename );
}

void PhoneApplet::modemFileDialog() {
    QString path = "/dev";
    QString filter = "tty*|Serial devices\n*|All files";
    QString filename = KFileDialog::getOpenFileName( KUrl( path ),
            filter, m_AppletSetup, name() );
    if  ( filename != "" )
        m_AppletSetup->UI.lineEdit_Modem->setText( filename );
}

QString PhoneApplet::faxFileDialog() {
    QString path = getenv( "HOME"  );
    //QString filter = "text/* image/fax-g3";
    //filter += "\napplication/postscript application/pdf image/x-eps";
    //filter += "\n*/*";
    QString filter = "*|All files";
    QStringList filenames = KFileDialog::getOpenFileNames( KUrl( path ),
            filter, 0, name() );
    return filenames.join(" ");
}

void PhoneApplet::digitClicked( char digit) {
    m_PhoneNumber->UI.lineEdit_Number->insert( QString( digit ) );

    if  ( m_eState == eTALK ) {
        m_PostConnectThread->add( digit );
    }
    else {
        m_PhoneNumber->clearStatusLine();
    }
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
}

void PhoneApplet::digitClickedFlsh() {
    if  ( m_eState == eTALK ) {
        m_PostConnectThread->add( '!');
    }
    else if ( m_eState == eIDLE || m_eState == eRING ) {
        AnswerCall();
    }
    else {
        m_PhoneNumber->clearStatusLine();
    }

    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
}

void PhoneApplet::deleteText() {
    if  ( m_PhoneNumber->unadoptWidget() )
        return;
    if  ( m_bUseSearch ) {
        useSearch();
        return;
    }

    if  ( m_PhoneNumber->UI.lineEdit_Number->cursorPosition() >= m_PhoneNumber->number().length() )
        m_PhoneNumber->UI.lineEdit_Number->backspace();
    else
        m_PhoneNumber->UI.lineEdit_Number->del();
    
    if  ( m_eState != eTALK )
        m_PhoneNumber->clearStatusLine();
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
}

void PhoneApplet::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect) {
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing );
    // Now we draw the applet, starting with our svg for appropriate program state
    // The icons do not look good filling the whole rectangle so:
    //QRect myRect;
    //QPoint myOffset;

    //myOffset.setX( contentsRect.width() / 10 );
    //myOffset.setY( contentsRect.height() / 10 );
    //myRect = contentsRect;
    //myRect.setX( contentsRect.x() + myOffset.x() );
    //myRect.setY( contentsRect.y() + myOffset.y() );
    //myRect.setWidth( contentsRect.width() - 2 * myOffset.x() );
    //myRect.setHeight( contentsRect.height() - 2 * myOffset.y() );

    Qt::Alignment align = Qt::AlignCenter;
    QIcon::Mode mode = m_bMouseOver ? QIcon::Active : QIcon::Normal;

    switch ( m_eState ) {
        case eTALK:
            if  ( m_bAlternate )
                m_iconTalk2.paint( p, contentsRect, align, mode );
            else
                m_iconTalk.paint( p, contentsRect, align, mode );
            break;
        case eRING:
            if  ( m_bAlternate )
                m_iconRing2.paint( p, contentsRect, align, mode );
            else
                m_iconRing.paint( p, contentsRect, align, mode );
            break;
        case eCONNECT:
            if  ( m_bAlternate )
                m_iconConnect2.paint( p, contentsRect, align, mode );
            else
                m_iconConnect.paint( p, contentsRect, align, mode );;
            break;
        case eDIAL:
            if  ( m_bAlternate )
                m_iconDial2.paint( p, contentsRect, align, mode );
            else
                m_iconDial.paint( p, contentsRect, align, mode );
            break;
        case eHANGUP:
            if  ( m_bAlternate )
                m_iconHangup2.paint( p, contentsRect, align, mode );
            else
                m_iconHangup.paint( p, contentsRect, align, mode );
            break;
        case eNOGOOD:
            if  ( m_bAlternate )
                m_iconNogood2.paint( p, contentsRect, align, mode );
            else
                m_iconNogood.paint( p, contentsRect, align, mode );
            break;
        case eSENDFAX:
            if  ( m_bAlternate )
                m_iconSendFax2.paint( p, contentsRect, align, mode );
            else
                m_iconSendFax.paint( p, contentsRect, align, mode );
            break;
        case eRECVFAX:
        case ePICKUPFAX:
            if  ( m_bAlternate )
                m_iconRecvFax2.paint( p, contentsRect, align, mode );
            else
                m_iconRecvFax.paint( p, contentsRect, align, mode );
            break;
        default:
            m_iconMain.paint( p, contentsRect, align, mode );
            break;
    }
}

void PhoneApplet::paintInterfaceState(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect,
        Plasma::Svg& svg) {
    svg.resize((int) contentsRect.width(), (int) contentsRect.height());
    svg.paint(p, (int) contentsRect.left(), (int) contentsRect.top());
}

void PhoneApplet::StartDaemon() {
    if  ( m_DaemonThread->isRunning() )
        return;

    m_DaemonThread->setModemName( m_strModemFilename );
    m_DaemonThread->start();
    sleep(1);
    if  ( !m_DaemonThread->isRunning() ) {
        set_state( eNOGOOD );
        int err = m_DaemonThread->error();
        ErrMessage( err < 0 ? err : ERRNO_NODAEMON );
        return;
    }

    set_state( eIDLE );
}

void PhoneApplet::set_state( eMODEM_STATE newstate ) {
    m_bAlternate = FALSE;
    m_eState = newstate;
    
    // phone status icon
    switch ( m_eState ) {
        case eCONNECT:
        case eDIAL:
        case eHANGUP:   m_PhoneNumber->UI.toolButton_Status0->setIcon( KIcon( "view-history" ) );   break;
        case eTALK:     m_PhoneNumber->UI.toolButton_Status0->setIcon( KIcon( "audio-headset" ) );  break;
        case eRECVFAX:
        case ePICKUPFAX:
        case eSENDFAX:  m_PhoneNumber->UI.toolButton_Status0->setIcon( KIcon( "document-print-preview" ) );        break;
        default:        m_PhoneNumber->UI.toolButton_Status0->setIcon( KIcon() );                   break;
    }

    if (m_eState == eTALK || m_eState == eSENDFAX || m_eState == eRECVFAX  || m_eState == ePICKUPFAX ) {
        m_PhoneNumber->resetTimeElapsed();
        m_QTimerSeconds->start();
    } else {
        m_QTimerSeconds->stop();
    }

    if  ( m_eState == eIDLE )
        m_QTimerBlink->stop();
    else
        m_QTimerBlink->start();
    update();
    setConnectIcon();
}

void PhoneApplet::blinkit() {
    m_bAlternate = !m_bAlternate;
    update();
    setConnectIcon();
}

void PhoneApplet::AbortDaemon() {
    if  ( !m_DaemonThread->isRunning() )
        return;

    m_DaemonThread->abort();
    //m_DaemonThread->terminate();        // does not work
    //m_DaemonThread->wait( MSEC_TWO_SECONDS );
}

// FOR m_iModemFileno ONLY
void PhoneApplet::modem_close() {
    if  ( !m_bWantKeep )
        m_modem->close();
}

void PhoneApplet::proc_ring() {
    if  ( m_eState == eDIAL ) {
        /*m_DialThread->abort();
        modem_close();
        ErrMessage( ERRNO_TIMEOUT );
        set_state( eRING );*/
    }
    else if  ( m_eState == eIDLE ) {
        set_state( eRING);
    }
    else {}

    m_QTimerEndRings->start();

    if (m_eState == eRING && m_bWantSound)
        play_ring( m_strPlaySound, m_strSoundFile );
}

void PhoneApplet::play_ring( QString player, QString sound ) {
    // play the sound, do not care about restult
    //QtConcurrent::run( fork_cmd, m_strPlaySound );
    //QtConcurrent::run( QSound::play, PLAY_SOUND_FILE );
    QString cmd = player;
    if  ( cmd.contains( CMDARG1 )) {
        cmd.replace( CMDARG1, sound );
    }
    else {
        cmd += ' ';
        cmd += sound;
    }
    m_playSound.startDetached(cmd);
}

void PhoneApplet::test_ring() {
    play_ring( m_AppletSetup->UI.lineEdit_Player->text(),
            m_AppletSetup->UI.lineEdit_Sound->text() );
}

void PhoneApplet::proc_endringseq() {
    if  ( m_eState == eCONNECT && !m_AnswerThread->isFinished() ) {
        set_state( eIDLE );
        m_AnswerThread->abort();
        modem_close();
        ErrMessage( ERRNO_TIMEOUT );
        return;
    }
    if  ( m_eState == eRING )
        set_state( eIDLE );
}

void PhoneApplet::toggle_sound() {
    //m_bWantSound = !m_bWantSound;
    m_bWantSound = m_PhoneNumber->UI.toolButton_Bell->isChecked();
    setBellIcon();
    saveConfig();     // do this immediately!
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
}

void PhoneApplet::clickVolume() {
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
    
    if  ( m_eState == eDIAL || m_eState == eCONNECT || m_eState == eHANGUP )
        return; // not the time!
    
    int newVolume;
    
    switch (m_iVolume) {
        case 0: newVolume = 1;
            m_bVolumeRising = TRUE;
            break;
        case 3: newVolume = 2;
            m_bVolumeRising = FALSE;
            break;
        case 1:
        case 2: if (m_bVolumeRising)
                newVolume =m_iVolume + 1;
            else
                newVolume = m_iVolume -1;
            break;
        default: return;
    }

    if  ( m_eState == eTALK )
        m_PostConnectThread->add( newVolume ); // setVolume() on success
    else
        setVolume( newVolume );
}

void PhoneApplet::setVolume( int newVolume ) {
    m_iVolume = newVolume;
    setVolumeIcon();
    saveConfig();  // do this immediatly/
}

void PhoneApplet::setVolumeIcon() {
    switch ( m_iVolume ) {
        case 3:     m_PhoneNumber->UI.toolButton_Status1->setIcon( KIcon( "audio-volume-high"));    break;
        case 2:     m_PhoneNumber->UI.toolButton_Status1->setIcon( KIcon( "audio-volume-medium"));  break;
        case 1:     m_PhoneNumber->UI.toolButton_Status1->setIcon( KIcon( "audio-volume-low"));     break;
        case 0:     m_PhoneNumber->UI.toolButton_Status1->setIcon( KIcon( "audio-volume-muted"));   break;
        default:    break;
    }
}

void PhoneApplet::setConnectIcon() {
    int con = m_modem->connection();
    // more of a diagnostic
    m_PhoneNumber->UI.toolButton_Status3->setIcon( con > 0 ? KIcon( "network-wired") : KIcon() );
    m_PhoneNumber->UI.toolButton_Status3->setPalette( con >    4
                                                        ? QColor( 150, 255, 127 ) // green, pastelle
                                                    : con > 3
                                                        ? QColor( 200, 255, 0 ) // yellow/green
                                                    : con > 2
                                                        ? QColor( 255, 255, 127 ) // yellow
                                                    : con > 1
                                                        ? QColor( 255, 200, 0 ) // orange
                                                    : con > 0
                                                        ? QColor( 255, 170, 200 ) // red
                                                        : Qt::white
            );

}
void PhoneApplet::setBellIcon() {
    m_PhoneNumber->UI.toolButton_Status2->setIcon( m_bWantSound ? KIcon( "preferences-desktop-notification-bell" ) : KIcon() );//KIcon("user-busy") );
}

void PhoneApplet::setDaemonIcon()  {
    m_PhoneNumber->UI.toolButton_Status4->setIcon( ( m_DaemonThread->isRunning() )
                                                            ? KIcon( "task-recurring" ) : KIcon() );
}

void PhoneApplet::toggle_daemon() {
    if  ( m_DaemonThread->isRunning() )
        AbortDaemon();
    else
        m_DaemonThread->start();
}

// This is the UI:
void PhoneApplet::mousePressEvent ( QGraphicsSceneMouseEvent *ev ) {
    if  ( signalsBlocked() ) {
        Plasma::Applet::mousePressEvent( ev );
        return;
    }

    if  ( ev->button() == Qt::LeftButton )
        proc_click();
    else
        Plasma::Applet::mousePressEvent( ev );
}

void PhoneApplet::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    //raise();
    m_bMouseOver = TRUE;
    update();
    Plasma::Applet::hoverEnterEvent( event );
}

void PhoneApplet::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    //lower();
    m_bMouseOver = FALSE;
    update();
    Plasma::Applet::hoverLeaveEvent( event );
}

void PhoneApplet::dragEnterEvent(QGraphicsSceneDragDropEvent* event) {
    if (event->mimeData()->hasText()) {
        event->setDropAction(Qt::CopyAction);   // gives me cursor!
        event->accept();
    }
    else {
        event->ignore();
    }

}

void PhoneApplet::dragMoveEvent(QGraphicsSceneDragDropEvent* event) {
    if (event->mimeData()->hasText()) {
        event->accept();                    // must do this
    }
    else {
        event->ignore();
    }

}

void PhoneApplet::dropEvent(QGraphicsSceneDragDropEvent *event) {
    if (event->mimeData()->hasText()) {
        m_PhoneNumber->setNumber(event->mimeData()->text());
        if (!m_PhoneNumber->isVisible())
            ToggleGUI();
        if (m_eState == eIDLE)
            DialCall();
    }
    else {
        event->ignore();
    }
}

void PhoneApplet::HangUp( bool error, bool fromdial ) {
    if  ( m_modem->fn() < 1 ) {
        // should not happen
        ErrMessage( ERRNO_NOMODEM );
        return;
    }

    set_state( eHANGUP );

    m_HangupThread->setFromDial( fromdial );
    m_HangupThread->setForceReset( error );
    m_HangupThread->start();
    //m_DaemonThread->pause(MSEC_WAIT_THREAD, m_HangupThread);
}

void PhoneApplet::CallHungUp() {
    if  ( !m_bWantKeep || m_HangupThread->forcedReset() || !m_HangupThread->result() )
        modem_close();
    //m_PhoneNumber->hide();
    set_state( eIDLE );
}

void PhoneApplet::AnswerCall() {
    if  ( m_modem->openSession() < 0 ) {
        ErrMessage( ERRNO_NOMODEM );
        return;
    }

    //toggle_daemon();
    set_state( eCONNECT );

    m_AnswerThread->start();
    //m_DaemonThread->pause( MSEC_WAIT_THREAD, m_AnswerThread );
    m_QTimerEndRings->stop();
}

void PhoneApplet::DialCall() {
    // prevent recursion
    if (m_DialThread->isRunning())
        return;

    int digit;
    bool follows;

    if  ( isQuick( &digit, &follows ) ) {
        // # sequence #0 .. #9, dial it
        if  ( digit >= 0 && !follows ) {
            quickClicked( digit );
            return;
        }
        // if anything follow, try to dial
        if  ( !follows )
            return;
    }

    QString number = m_PhoneNumber->number();

    if  ( number.length() < 1 ) {
        // get head history item and place in text, do nothing else  :-)
        //if  ( m_callsMade->count()  > 0 )
        //    m_PhoneNumber->setNumber( historyHead() );
        m_iInHistory = 0;
        if  (!m_PhoneNumber->isAdopted() )
            startHistoryList();
        return;
    }

    historyAdd( number );
    doDial( MyListWidget::numberOfEntry( number ) );
}

void PhoneApplet::doDial(QString number) {
    if  ( m_modem->openSession() < 0 ) {
        ErrMessage(ERRNO_NOMODEM);
        return;
    }
    
    number.remove( ' ' ); //carefull!
    number.remove( '-' );
    number.remove( '(' );
    number.remove( ')' );

    //toggle_daemon();
    set_state(eDIAL);
    //MessageBox ( m_PhoneNumber->number().toAscii(), FALSE );
    m_DialThread->setNumber( number );
    m_DialThread->start();
    //m_DaemonThread->pause(MSEC_WAIT_THREAD, m_DialThread);
}

bool PhoneApplet::isQuick(int* digit, bool* numbfollows) {
    *digit = -1; // none
    *numbfollows = FALSE;

    QString number = m_PhoneNumber->number();
    int len = number.length();
    if  ( len < 1 )
        return FALSE;

    if (number.at(0) == QChar('#')) {
        if (len > 1 && number.at(1).isDigit()) {
            *digit = number.at(1).digitValue();
            *numbfollows = len > 2;
        }
        return TRUE;
    }

    return FALSE;
}

QString PhoneApplet::quickData() {
    return m_PhoneNumber->number().mid( 2 ).trimmed();
}

QString PhoneApplet::quickNumber() {
    return MyListWidget::numberOfEntry( quickData() );
}

QString PhoneApplet::quickNumber( int ix ) {
    return MyListWidget::numberOfEntry( m_quickDial[ ix ] );
}

QString PhoneApplet::quickNumber( QString data ) {
    return MyListWidget::numberOfEntry( data );
}

void PhoneApplet::RedialCall() {
    if  ( m_eState != eIDLE)
        return;
    if  ( m_callsMade->count()  < 1 )
        return;
    if  ( m_DialThread->isRunning() )
        return;

    m_PhoneNumber->setNumber( historyHead() );
    doDial( MyListWidget::numberOfEntry( historyHead() ) );
}

void PhoneApplet::quickClicked( int ix, bool pn ) {
    m_PhoneNumber->unadoptWidget();

    QString number;
    if  ( pn ) {
        ix += m_QuickSetup->currentIndex() * NQUICK;
        number = m_quickEdit[ ix ]->text();
    }
    else {
        number = m_quickDial[ ix ];
    }

    if  ( number == "" )
        return;
    m_PhoneNumber->setNumber( number );

    if  ( m_eState != eIDLE)
        return;
    if  ( m_DialThread->isRunning() )
        return;

    historyAdd( number );
    doDial( MyListWidget::numberOfEntry( number ) );
}

void PhoneApplet::quickListClicked( const QModelIndex& qix ) {
    int row = qix.row();
    if  ( m_quickDial[ row ].length() > 0 )
        quickClicked( row );
}

void PhoneApplet::quickListSelect( int ix ) {
    if  ( ix < 0 || ix >= m_quickDial.count())
        return;
    
    QString number = m_quickDial[ ix ];
    if  ( number == "" )
        return;
    m_PhoneNumber->setNumber( number );
}

QString PhoneApplet::historyHead() {
    return m_callsMade->historyItems()[0];
}

void PhoneApplet::historyAdd( QString number ) {
    m_callsMade->addToHistory( number );    
    saveConfig();           // do immediately
}

void PhoneApplet::clickNext() {
    int cnt;

    // in existing adopted widget
    if (m_PhoneNumber->isAdopted()) {
        if ( m_PhoneNumber->adopted() == m_contactsList ) {
            m_contactsList->nextRow();
        }        
        else if (m_PhoneNumber->adopted() == m_callsList) {
            m_callsList->nextRow();
        }
        else { }
        return;
    }

    // quick-dial
    int digit;
    bool follows;
    if  ( isQuick( &digit, &follows) ) {
        if  ( digit >= 0 && follows ) {
            // add this number
            m_contactsList->delQuickContact( digit, m_quickDial[ digit ] );
            m_quickDial[ digit ] = quickData();
            if  ( m_bMergeQuick )
                m_contactsList->addQuickContact( digit, m_quickDial[ digit ] );
            saveConfig();
        }
        startQuickList( digit );
        return;
    }

    if (m_bUseSearch) {
        // doing this as a circular subsearch
        QString text = m_PhoneNumber->number();
        cnt = m_contactsList->count();
        if (cnt < 1)
            return;

        doTextEdited("");
        m_contactsList->nextRow();
        doTextEdited(text);
    }
    else if  ( m_callsMade->count() > 0 ) {
        startHistoryList();
    }
    else { }
}

void PhoneApplet::clickPrev() {
    // existing adopted widget
    if  ( m_PhoneNumber->isAdopted() ) {
        if ( m_PhoneNumber->adopted() == m_contactsList ) {
            m_contactsList->prevRow();
        }
        else if (m_PhoneNumber->adopted() == m_callsList) {
            m_callsList->prevRow();
        }
        else { }
        return;
    }

    // quick-dial
    int digit;
    bool follows;
    if  ( isQuick( &digit, &follows) ) {
        startQuickList( digit );
        return;
    }

    if  ( m_bUseSearch ) {
        useSearch();
        startContacts();
    }
    else if  ( m_callsMade->count() > 0 ) {
        startHistoryList();
    }
    else { }
}

void PhoneApplet::startHistoryList() {
    if  ( m_callsMade->count() < 1 )
        return;

    int save = m_iInHistory;    // side effect from below??

    if (!m_PhoneNumber->adopted()) {
        m_callsList->disconnect();
        m_callsList->clear();
        m_callsList->addItems(m_callsMade->historyItems());
        connect(m_callsList, SIGNAL(currentRowChanged(int)), this, SLOT(historySelect(int)));
        connect(m_callsList, SIGNAL(clicked(const QModelIndex&)), this,
                                SLOT(historyClick(const QModelIndex&)));
        m_PhoneNumber->adoptWidget(m_callsList);
    }

    m_iInHistory = save;
    m_callsList->setRow( m_iInHistory );
    m_PhoneNumber->setNumber( m_callsMade->historyItems()[ m_iInHistory ] );
}

void PhoneApplet::historySelect( int ix ) {
    if  ( ix < 0 || ix >= m_callsMade->count() )
        return;
    
    m_iInHistory = ix;
    m_PhoneNumber->setNumber( m_callsMade->historyItems()[ ix ] );
}

void PhoneApplet::historyClick( const QModelIndex& qix ) {
    historySelect( qix.row() );
    m_PhoneNumber->unadoptWidget();
    if  ( m_eState == eIDLE )
        DialCall();    
}

void PhoneApplet::contactsSelect( int ix ) {
    if  ( ix < 0 || ix >= m_contactsList->count() )
        return;

    m_iInContacts = ix;
    if  ( m_bUseSearch || m_PhoneNumber->adopted() )
        m_PhoneNumber->setNumber( m_contactsList->lineItem( ix ) );
}

void PhoneApplet::contactsClick( const QModelIndex& qix ) {
    contactsSelect( qix.row() );
    m_PhoneNumber->unadoptWidget();
    if  ( m_eState == eIDLE )
        DialCall();
}

void PhoneApplet::startQuickList( int digit ) {
    if (!m_PhoneNumber->adopted()) {
        m_callsList->disconnect();
        m_callsList->clear();
        int nq = m_quickDial.count();
        for ( int q = 0; q < nq; ++q ) {
            QString quick;
            if  ( q < NQUICK )
                QTextStream( &quick ) << "#";
            QTextStream( &quick ) << q << ' ' << m_quickDial[ q ];
            m_callsList->addItem( quick );
        }
        connect(m_callsList, SIGNAL(currentRowChanged(int)), this, SLOT(quickListSelect(int)));
        connect(m_callsList, SIGNAL(clicked(const QModelIndex&)), this,
                                SLOT(quickListClicked(const QModelIndex&)));
        m_PhoneNumber->adoptWidget(m_callsList);
    }

    m_iInQuick = digit < 0
                    ? 0
            : digit >= NQUICK
                    ? NQUICK - 1
                    : digit;

    m_callsList->setRow( m_iInQuick );
}

void PhoneApplet::abortDial() {
    m_DialThread->abort();
}

void PhoneApplet::timeElapsed() {
    if  ( m_eState == eSENDFAX && m_SendFaxThread->isRunning() ) {
        m_PhoneNumber->setPrefix( Message( m_SendFaxThread->converting()
                                ? ERRNO_FAXCONVERTING : ERRNO_FAXSENDING ) );
        // do this so empty box does not show
        int lbef = m_InfoBox->text().length();
        QString rpt = m_SendFaxThread->output();
        m_InfoBox->setText( m_InfoBox->text() + rpt );
        if  ( lbef < 1 && rpt.length() > 0 )
            m_InfoBox->setVisible( m_bWantFaxOutput );
    }
    if  ( ( m_eState == eRECVFAX || m_eState == ePICKUPFAX) && m_RecvFaxThread->isRunning() ) {
        m_PhoneNumber->setPrefix( Message( ERRNO_FAXRECVING ) );
        // do this so empty box does not show
        int lbef = m_InfoBox->text().length();
        QString rpt = m_RecvFaxThread->output();
        m_InfoBox->setText( m_InfoBox->text() + rpt );
        if  ( lbef < 1 && rpt.length() > 0 )
            m_InfoBox->setVisible( m_bWantFaxOutput );
    }

    m_PhoneNumber->setTimeElapsed();
}

void PhoneApplet::abortConnectThreads() {
    m_PostConnectThread->abort();
}

void PhoneApplet::CallAnswered() {
    bool stat, aborted;
    int err;

    //toggle_daemon();
    switch ( m_eState ) {
        case eCONNECT:
            stat = m_AnswerThread->result();
            err = m_AnswerThread->error();
            aborted = m_AnswerThread->aborted();
            break;
        case eDIAL:
        case eHANGUP:   // could get here is dialing thread is aborted!
            stat = m_DialThread->result();
            err = m_DialThread->error();
            aborted = m_DialThread->aborted();
            break;
        default:
            set_state( eIDLE );
            return;
    }

    if  ( !stat ) {
        //m_PhoneNumber->setVisible( FALSE );
        HangUp( TRUE, FALSE );    
        ErrMessage( err );
        return;
    }

    if  ( aborted )
        return;

    set_state( eTALK );
    //m_PhoneNumber->setVisible( TRUE );
    m_PostConnectThread->start();
}

void PhoneApplet::ToggleGUI() {
    m_PhoneNumber->move( popupPosition( m_PhoneNumber->size() ) );
    m_PhoneNumber->setVisible( !m_PhoneNumber->isVisible() );
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
}

void PhoneApplet::proc_click() {
    if  ( m_QTimerMouse->isActive() )
        return;

    //m_bMousePress = TRUE;
    //update();

    m_QTimerMouse->start();

    m_PhoneNumber->unadoptWidget();
    m_PhoneNumber->clearStatusLine();
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);

    switch (m_eState) {
        case eTALK: HangUp( FALSE, FALSE );
            break;
        case eRING: AnswerCall();
            break;
        case eIDLE: ToggleGUI();
            break;
        case eDIAL:
            HangUp( FALSE, TRUE );
            break;
        case eSENDFAX:
        case eRECVFAX:
        case ePICKUPFAX:
            cancelFax();
            break;
        case eNOGOOD:
            showConfigurationInterface();
            break;
        default: break;
    }
}

void PhoneApplet::UIButtonClick() {
    // actions like main click
    if  ( m_QTimerMouse->isActive() )
        return;
    m_QTimerMouse->start();

    m_PhoneNumber->unadoptWidget();
    m_PhoneNumber->clearStatusLine();
    m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);

    switch (m_eState) {
        case eTALK: HangUp( FALSE, FALSE );
            break;
        case eRING: AnswerCall();
            break;
        case eDIAL: 
            HangUp( FALSE, TRUE );
            break;
        case eIDLE:
            if  ( m_bUseSearch )
                useSearch();    // will cancel status
            DialCall();
            break;
        case eSENDFAX:
        case eRECVFAX:
        case ePICKUPFAX:
            cancelFax();
            break;
        default: break;
    }
}

int PhoneApplet::proc_dlecommand(char* dlecmd ) {
    //QString cmd = "<DLE>";

    char *dlepos = strcasestr( dlecmd, MODEM_DLE );
    if  ( dlepos == NULL)
        return 0;   // nothing to do

    char code = *(dlepos + strlen( MODEM_DLE ));

    if  ( code == 'b' )
        return ERRNO_BUSY;
    
    // future use of DTMF stuff here
    
    
    return 0;
}

void PhoneApplet::PostConnectEnd() {
    if  ( m_eState != eTALK )
        return;
    if  ( m_PostConnectThread->aborted() )
        return; // avoid recursion
    if  ( m_PostConnectThread->result() )
        return; // peaceful conclusion :-)

    int err = m_PostConnectThread->error();
    ErrMessage( err );
    if  ( err != ERRNO_MODEMVOLUME && err != ERRNO_DTMF && !m_HangupThread->isRunning() )
        HangUp( TRUE, FALSE );
}

void PhoneApplet::ErrMessage( int err_no ) {
    MessageBox( Message( err_no ) );
}

void PhoneApplet::MessageBox( const QString cmd, bool error ) {
    if  ( m_PhoneNumber->isVisible() ) {
        QString qcmd = "<html>";
        if  ( error )
            qcmd = "<font color=red>";
        else
            qcmd = "<font color=green>";
        qcmd += cmd;
        qcmd += "</font></html>";
        m_PhoneNumber->setStatusLine( qcmd );
        m_PhoneNumber->UI.lineEdit_Number->setFocus(Qt::OtherFocusReason);
    }
    else {
        if  ( error )
            QMessageBox::critical( NULL, name(),  cmd );
        else
            QMessageBox::warning( NULL, name(), cmd );
    }
}


void PhoneApplet::dataUpdated( const QString& sourceName,  const Plasma::DataEngine::Data& data) {
    if (sourceName.contains(CONTACT_COLLECTIONS, Qt::CaseInsensitive)) {
        // load the contacts to engine
        QStringList keys = data.uniqueKeys();
        int nKeys = keys.count();

        for (int n = 0; n < nKeys; ++n) {
            if (keys[n].contains(CONTACT_COLLECTION_PATTERN, Qt::CaseInsensitive)) {
                m_eContactsData = eLOADING;
                m_Contacts->connectSource(keys[n], this, 0);
            }
        }
        return;
    }
    if  ( !sourceName.contains(ONE_CONTACT_PATTERN, Qt::CaseInsensitive ) )
            return;

    m_contactsList->addContact( data );
    m_eContactsData = eDONE;
    m_QTimerContacts->start();
}

void PhoneApplet::contactsOps() {
    switch ( m_eContactsData ) {
        case eNONE: 
            m_Contacts->connectSource( CONTACT_COLLECTIONS, this, 0 );
            break;
        case eLOADING:
        // try to get contacts until success :-)
            m_Contacts->connectAllSources( this, 0 );
            break;
        case eDONE:
            dataDone(); // simply get here after a delay!
            break;
        default:
            break;
    }

}

void PhoneApplet::dataDone() {
    m_PhoneNumber->UI.toolButton_Contacts->setEnabled( TRUE );
    m_QTimerContacts->stop();
}

void PhoneApplet::startContacts() {
    if  ( m_eContactsData == eLOADING )
        return;
    if  ( m_contactsList->count() < 1 )
        return;

    if  (!m_PhoneNumber->adopted()) {
        m_PhoneNumber->adoptWidget(m_contactsList);
    }

    m_contactsList->setRow( m_iInContacts );
    // because setCurrentRow might no register as new select
    contactsSelect( m_iInContacts );
}

void PhoneApplet::doTextEdited( QString text ) {
    m_bUseSearch = FALSE;

    // PROTECT this function:
    if  ( m_eContactsData == eLOADING )
        return;

    // test if I want this
    if  ( !m_bWantSearch )
        return;
    if  ( m_eState != eIDLE )
        return;     // because status may contain times

    // test can I do this
    //if  ( !m_bContactsData )
    //    return;
    if  ( m_contactsList->count() < 1 )
        return;

    // anything proer to do?
    if  ( text.length() < 1 ) {
        cancelSearch();
        return;
    }
    if  ( !text[0].isLetter() ) {
        cancelSearch();
        return;
    }

    m_contactsList->setRow( m_iInContacts );
    // because setCurrentRow might no register as new select
    contactsSelect( m_iInContacts );
    
    QString found = m_contactsList->richSearchFor( text );
    if  ( found.length() >  0 ) {
        m_PhoneNumber->setStatusLine( found );
        //m_PhoneNumber->UI.toolButton_Status0->setIcon( KIcon( "face-smile") );
        m_bUseSearch = TRUE;
    }
    else {
        cancelSearch();
    }
}

void PhoneApplet::doTextChanged( QString text ) {
    // Adjust the fone size to facilitate somewhat longer strings:
    int size;
    int len = text.length();
    if  ( len > 21 )
        size = 7;
    else if ( len > 17 )
        size = 8;
    else if ( len > 14 )
        size = 9;
    else
        size = 10;
    QFont font = m_PhoneNumber->UI.lineEdit_Number->font();
    font.setPointSize( size );
    m_PhoneNumber->UI.lineEdit_Number->setFont( font );
}

void PhoneApplet::cancelSearch() {
    m_bUseSearch = FALSE;
    //m_PhoneNumber->UI.toolButton_Status0->setIcon(KIcon());
    m_PhoneNumber->clearStatusLine();
}

void PhoneApplet::useSearch() {
    if  ( !m_bWantSearch )
        m_bUseSearch = FALSE;
    if  ( !m_bUseSearch )
        return;

    contactsSelect( m_iInContacts );
    cancelSearch();     //one shot!
}

void PhoneApplet::createConfigurationInterface(KConfigDialog *parent) {
    parent->setButtons( KDialog::Ok |  KDialog::Cancel | KDialog::Reset );
    
    newAppletSetup();
    newQuickListSetup();
    resetConfigDialog();

    KIconLoader kil;

    parent->addPage( m_AppletSetup, tr( "Applet Setup" ), m_PhoneNumber->ConfigPath );
    parent->addPage( m_QuickSetup, tr( "#Quick-Dial Numbers" ), kil.iconPath( "phone" ,
                        KIconLoader::Toolbar) );
    parent->addPage( m_FaxSetup, tr( "Fax Setup" ), kil.iconPath( "document-print-preview",
                        KIconLoader::Toolbar) );

    connect( parent, SIGNAL( okClicked()), this, SLOT( saveConfigDialog() ));
    connect( parent, SIGNAL( applyClicked()), this, SLOT( saveConfigDialog() ));
    connect( parent, SIGNAL( resetClicked()), this, SLOT( resetConfigDialog()));
    connect( parent, SIGNAL( cancelClicked()), this, SLOT( endConfigDialog()));
}

void PhoneApplet::doFax() {
    switch ( m_eState ) {
        case eIDLE: {
            m_PhoneNumber->unadoptWidget();
            // prevent recursion
            if (m_SendFaxThread->isRunning())
                return;

            int digit;
            bool follows;
            QString number;

            if (isQuick(&digit, &follows)) {
                // # sequence #0 .. #9, dial it
                if (digit < 0 || !follows)
                    return;
                number = quickNumber( digit );
            }
            else {
                if  ( m_bUseSearch )
                    useSearch();    // will cancel status
                number = m_PhoneNumber->number();
            }
            if (number.length() < 1)
                return;

            m_PhoneNumber->unadoptWidget();

            // 4. Get files to send, multiple selection
            QString files = faxFileDialog();
            if (files.length() < 1)
                return;

            QString cmd = m_strFaxSendCmd;
            cmd.replace(CMDARG1, m_strModemFilename);
            cmd.replace(CMDARG2, MyListWidget::numberOfEntry( number) );
            //cmd.replace( CMDARG3, files ); this done in thread!!

            set_state(eSENDFAX);

            m_InfoBox->setText("");

            m_SendFaxThread->setFiles(files);
            m_SendFaxThread->setCmd(cmd);
            m_SendFaxThread->setConv(m_strFileToFaxCmd);
            m_SendFaxThread->start();

            historyAdd(number);
            return;
        }

        case eRING:
        case eTALK: {
            m_PhoneNumber->unadoptWidget();
            // prevent recursion
            if (m_RecvFaxThread->isRunning())
                return;

            QString cmd = m_eState == eTALK
                                ? m_strFaxPickupCmd
                                : m_strFaxRecvCmd;
            cmd.replace(CMDARG1, m_strModemFilename );

            set_state( m_eState == eTALK ? ePICKUPFAX : eRECVFAX );

            m_InfoBox->setText("");

            m_RecvFaxThread->setPrefix( m_strFaxPrefix );
            m_RecvFaxThread->setCmd(cmd);
            m_RecvFaxThread->setViewer( m_strFaxViewCmd );
            m_RecvFaxThread->start();
            return;
        }

        default:
            return;
    }
}

void PhoneApplet::faxSendDone() {
    m_InfoBox->setText( m_InfoBox->text() + m_SendFaxThread->output() );

    resetAfterFax();
    set_state( eIDLE );
    if  ( m_SendFaxThread->aborted() ) {
        ErrMessage( ERRNO_FAXCANCELED);
        return;
    }
    if  ( m_SendFaxThread->result() ) {
        ErrMessage( ERRNO_FAXSENT );
        return;
    }
    ErrMessage( m_SendFaxThread->error() ); // serviced there!
}

void PhoneApplet::faxRecvDone() {
    m_InfoBox->setText( m_InfoBox->text() + m_RecvFaxThread->output() );

    if  ( m_eState == ePICKUPFAX )
        HangUp( !m_RecvFaxThread->result(), FALSE );

    resetAfterFax();
    set_state( eIDLE );

    if  ( m_RecvFaxThread->aborted() ) {
        ErrMessage( ERRNO_FAXCANCELED);
        return;
    }
    if  ( m_RecvFaxThread->result() ) {
        ErrMessage( ERRNO_FAXSENT );
        return;
    }
    ErrMessage( m_RecvFaxThread->error() ); // serviced there!
}

// Since the fax programs make their own changes!
void PhoneApplet::resetAfterFax() {
    m_modem->setToOpen( TRUE );         // regardless
    if  ( m_modem->openSession() >= 0 )
        m_modem->Init();
    m_modem->setToOpen( !m_bWantKeep ); // reset to options
    modem_close();
}

void PhoneApplet::cancelFax() {
    if  ( m_eState == eSENDFAX )
        m_SendFaxThread->abort();
    else if ( m_eState == eRECVFAX || m_eState == ePICKUPFAX )
        m_RecvFaxThread->abort();
    else {};
}

void PhoneApplet::addQuickContacts() {
    for ( int q = 0; q < m_quickDial.count(); ++q )
        m_contactsList->addQuickContact( q, m_quickDial[ q ] );
}

void PhoneApplet::removeQuickContacts() {
    for ( int q = 0; q < m_quickDial.count(); ++q )
        m_contactsList->delQuickContact( q, m_quickDial[ q ] );
}

// transportation to message boxes (C++ vs java :-)
void PhoneApplet::InfoBox() {
    m_InfoBox->InfoBox();
}

void PhoneApplet::modemInitsBox() {
    m_InfoBox->modemInitsBox();
}

void PhoneApplet::modemResetBox() {
    m_InfoBox->modemResetBox();
}

void PhoneApplet::sendFaxCmdBox() {
    m_InfoBox->sendFaxCmdBox();
}

void PhoneApplet::recvDialFaxCmdBox() {
    m_InfoBox->recvFaxCmdBox( TRUE );
}

void PhoneApplet::recvPickupFaxCmdBox() {
    m_InfoBox->recvFaxCmdBox(FALSE);
}

void PhoneApplet::convertToFaxCmdBox() {
    m_InfoBox->convertToFaxCmdBox();
}

void PhoneApplet::faxPrefixBox() {
    m_InfoBox->faxPrefixBox();
}

void PhoneApplet::faxViewerBox() {
    m_InfoBox->faxViewerBox();
}

void PhoneApplet::contactsBox() {
    m_InfoBox->contactsBox();
}

/*void PhoneApplet::keyShortcutBox() {
    m_InfoBox->keyShortcutBox();
}*/

void PhoneApplet::closeInfoBox() {
    m_InfoBox->setVisible( FALSE );
}

#include "phoneapplet.moc"

