// phoneapplet.h definitions
//
// David Baron, November 2008
//
// Here we avoid loading the header multiple times
#ifndef PhoneApplet_HEADER
#define PhoneApplet_HEADER

// We need the Plasma Applet headers
#include <Plasma/Applet>
#include <Plasma/DataEngine>
#include <Plasma/Svg>
#include <QIcon>
#include <QtGui>

#include <QTimer>
#include <QProcess>
#include <QListWidget>
#include <QObject>
#include <QHash>

#include <khistorycombobox.h>

#ifndef _PHONEAPPLET_CONSTANTS
#define _PHONEAPPLET_CONSTANTS

// global program parameters:
// Edit this for your modem device!
const QString MODEM_FILENAME = "/dev/ttyS2";
// Edit these for you distro and preference:
const QString CMD_PLAY_SOUND = "/usr/bin/ogg123 -d alsa <1>";
const QString PLAY_SOUND_FILE = "/usr/share/sounds/KDE-Im-Nudge.ogg";
const QString CMD_FAX_SEND = "efax -d <1> -t <2> <3>";
const QString CMD_FAX_RECV = "efax -d <1> -r <3>";
const QString CMD_FAX_PICKUP = "efax -d <1> -w -r <3>";
const QString CMD_FILE2FAX = "convert <1> g3:<2>";
const QString CMD_FAX_VIEWER = "viewfax";
const QString CMD_FAX_PREFIX = "phoneapplet-fax";
const QString CMDARG1 = "<1>";      // make sure consistant above
const QString HTMLARG1 = "&lt;1&gt;";
const QString CMDARG2 = "<2>";
const QString HTMLARG2 = "&lt;2&gt;";
const QString CMDARG3 = "<3>";
const QString HTMLARG3 = "&lt;3&gt;";

// Config keys for these == DO NOT CHANGE ONCE YOU HAVE RUN THE APPLET
const QString CONFIG_GROUP_PROPERTIES = "Properties";
const QString CONFIG_GROUP_OPTIONS = "Options";
const QString CONFIG_GROUP_CALLS = "Calls";
const QString CONFIG_MODEM_FILENAME = "ModemName";
const QString CONFIG_MODEM_INITS = "ModemInits";
const QString CONFIG_PLAY_SOUND = "SoundPlayer";
const QString CONFIG_SOUND_FILE = "SoundFile";
const QString CONFIG_FAX_SEND = "FaxSend";
const QString CONFIG_FILE2FAX = "FileToFax";
const QString CONFIG_FAX_RECV = "FaxRecv";
const QString CONFIG_FAX_PICKUP = "FaxPickup";
const QString CONFIG_FAX_OUTBOX = "FaxOutBox";
const QString CONFIG_FAX_VIEWER = "FaxViewer";
const QString CONFIG_FAX_PREFIX = "FaxPrefix";
const QString CONFIG_WANT_SOUND = "SoundActive";
const QString CONFIG_MODEM_VOLUME = "ModemVolume";
const QString CONFIG_WANT_RESET = "ResetOnHangup";
const QString CONFIG_WANT_KEEP = "KeepModemOpen";
const QString CONFIG_QUICKDIAL = "QuickDialNumbers";
const QString CONFIG_CALLSMADE = "CallsMade";
const QString CONFIG_USE_ENGINE = "UseEngine";
const QString CONFIG_WANT_SEARCH = "KeyboardSearch";
const QString CONFIG_SEARCH_QUICK = "SearchQuick";

const QString ACTION_GUI = "GUI";
const QString ACTION_CONFIG = "CONFIG";
const QString ACTION_ABOUT = "ABOUT";

const QString CONTACT_ENGINE = "akonadi";
const QString CONTACT_COLLECTIONS = "ContactCollections";
const QString CONTACT_COLLECTION_PATTERN = "ContactCollection";
const QString ONE_CONTACT_PATTERN = "Contact";
const QString ONE_CONTACT_PHONE = "Phone";
const QString ONE_CONTACT_NAME = "Name";

// These are for Hayes comparable, Rockwell chip (336, etc) speakerphone modems
// Edit these for your modem if necessary
// Some modems use + instead of # and this is settable in a register if need be
const QString MODEM_INITS = "~AT S7=5 S6=5 S0=0 M2 V1 X4 &c1 E0 Q0";    // pref ~\r~
                            // S7 remote carrier wait 5// 45 secs
                            // S6 dialtone wait 5 secs
                            // S0 rings before auto-answer--no answwer
                            // M2 speaker always on (0-off, 1-till answer 3-during answer)
                            // V1 textual responses
                            // X4 all of the above
                            // &c1 DCD follows carrier
                            // E0 echo off
                            // Q0 DTR set
const char* MODEM_RESET = "ATZ";
const char* MODEM_ANSWER = "ATA";
const char* MODEM_HANGUP = "ATH";
const char* MODEM_TONE_DIAL_PRE = "ATDTW";
                            // Dial, tones, wait for dial-tone
const char* MODEM_TONE_DIAL_SUF = "";
const char* MODEM_ZERO_VOLUME = "ATL0";
const char* MODEM_LOW_VOLUME = "ATL1";
const char* MODEM_MED_VOLUME = "ATL2";
const char* MODEM_HI_VOLUME = "ATL3";
const char* MODEM_TONE = "AT#VTS=";
const char* MODEM_READ_S6 = "ATS6?";
const char* MODEM_CID = "AT#CID";

// should precede dialing or answer:
// may change for other models
const char* MODEM_SET_SPEAKERPHONE = "AT#CLS=8 #VRA=60 #VRN=0 #VLS=6";
                                    // CLS=8 voice mode
                                    // VRA=60 6 seconds ringback check
                                    // VRN=0 no no-ringback check
                                    // VLS=6 choose headset
const char* MODEM_SET_DTMF = "AT#VTD=3F,3F,3F";
                                    // ALL OF THEM
                                    // only third one relevant here

// standard? modem status returns
const char* MODEM_WAS_OK = "OK";
const char* MODEM_WAS_ERROR = "ERROR";
const char* MODEM_VOICE_COM = "VCON";
const char* MODEM_RING = "RING";
const char* MODEM_BUSY = "BUSY";
const char* MODEM_CARRIER = "CARRIER";
const char* MODEM_DIALTONE = "DIALTONE";
const char* MODEM_NOANSWER = "ANSWER";
const char* MODEM_DLE="\x10";       // byte-flag for DTMF receieved

// Edit these to fine-tune for your modem if need be
// Would want the highest that works, however something safe may be better
// This modem is most probably NOT being used for data transfer at this point
const speed_t MODEM_BAUD = B57600;          // might use something safe, this modem NOT used for data here

// millisecond timings
const long MSEC_IGNORE_RING = 500l;         // because rings can be irregular
const long MSEC_BLINK = 667l;               // blink time, to taste
const long MSEC_AFTER_RINGS = 3500l;        // if I do not answer the phone

// Maximum wait for response timeouts. These numbers should be OK
const long MSEC_READ_TIMEOUT = 250l;        // most modem commands
const long MSEC_DTMF_TIMEOUT = 600l;        // presumably includes the beep time and read timeout
const long MSEC_RESET_TIMEOUT = 3000l;     // hardware problems?
const long MSEC_DIAL_TIMEOUT = 20000l;      // voice modems cannot know when there is a voice answer
                                            // how long to wait before I assume answer may have happened
const long MSEC_DIAL_ADDTIMEOUT = 5000l;

const long MSEC_ONE_SECOND = 1000l;
const long MSEC_TWO_SECONDS = 2000l;
const long MSEC_MOUSE_LIMIT = 200l;         // Too-rapid or spontaneous double-clicks may foul things up

const int RETRIES = 3;
const int NQUICK=10;

// ERROR STATUSES AND MESSAGES -- will want to translate sometime :-)
const QString MESSAGES[] = { QObject::tr("Could not do it"),
                        QObject::tr("Could not open modem"),
                        QObject::tr("Incompatable Linux version"),
                        QObject::tr("Could not initialize modem"),
                        QObject::tr("Could not reset modem"),
                        QObject::tr("Could not set modem to speakerphone"),
                        QObject::tr("Could not hang up modem"),
                        QObject::tr("Line busy"),
                        QObject::tr("No carrier"),
                        QObject::tr("Could not succeed in time"),
                        QObject::tr("Could not answer"),
                        QObject::tr("No phone number entered"),
                        QObject::tr("Cannot dial"),
                        QObject::tr("No dial tone"),
                        QObject::tr("No answer"),
                        QObject::tr("Cannot set volume"),
                        QObject::tr("Error sending DTMF"),
                        QObject::tr("Error enabling recieve DTMF"),
                        QObject::tr("Device not modem"),
                        QObject::tr(""),
                        QObject::tr("Error sending fax"),
                        QObject::tr("Error receiving fax"),
                        QObject::tr("Error converting to fax format"),
                        QObject::tr("Fax succesfully sent"),
                        QObject::tr("Sending... "),
                        QObject::tr("Receiving..."),
                        QObject::tr("Converting..."),
                        QObject::tr("Fax canceled")
                    };
const int   ERRNO_NOMODEM =         -1;
const int   ERRNO_BADLINUX =        -2;
const int   ERRNO_INITMODEM =       -3;
const int   ERRNO_RESETMODEM =      -4;
const int   ERRNO_SETSPEAKERPHONE = -5;
const int   ERRNO_HANGUPMODEM =     -6;
const int   ERRNO_BUSY =            -7;
const int   ERRNO_NOCARRIER =       -8;
const int   ERRNO_TIMEOUT =         -9;
const int   ERRNO_ANSWERMODEM =     -10;
const int   ERRNO_NONUMBER =        -11;
const int   ERRNO_DIAL  =           -12;
const int   ERRNO_NODIALTONE =      -13;
const int   ERRNO_NOANSWER =        -14;
const int   ERRNO_MODEMVOLUME =     -15;
const int   ERRNO_DTMF =            -16;
const int   ERRNO_SETDTMF =         -17;
const int   ERRNO_NODAEMON =        -18;
const int   ERRNO_SENDFAXERROR =    -20;
const int   ERRNO_RECVFAXERROR =    -21;
const int   ERRNO_FAXCONVERT =      -22;
const int   ERRNO_FAXSENT =         -23;
const int   ERRNO_FAXSENDING =      -24;
const int   ERRNO_FAXRECVING =      -25;
const int   ERRNO_FAXCONVERTING =   -26;
const int   ERRNO_FAXCANCELED =     -27;
const int   MXERR =                 -27;
const int   ERRNO_GENERAL =         -100;
#endif

// These are the program states:

typedef enum {
    eIDLE, eRING, eCONNECT, eDIAL, eTALK, eHANGUP, eSENDFAX, eRECVFAX, ePICKUPFAX, eBUSY, eNOGOOD
} eMODEM_STATE;

typedef enum {
    eNONE, eLOADING, eDONE
} eCONTACTS_STATE;


class QSizeF;

class MyModem;
class RingDaemonThread;
class ModemAnswerThread;
class ModemHangupThread;
class ModemDialThread;
class PostConnectThread;
class SendFaxThread;
class RecvFaxThread;
class MyPhoneNumber;
class MyAppletSetup;
class MyFaxSetup;
class MyQuickSetup;
class InfoBoxes;
class MyListWidget;
class ContactsListWidget;

// Define our plasma Applet
namespace Plasma {
    class Svg;
    class DataEngine;
    class Package;
    class PackageMetaData;
}

class PhoneApplet : public Plasma::Applet {
    Q_OBJECT
            
public:
    // public functions
    // Basic Create/Destroy
    PhoneApplet(QObject *parent, const QVariantList &args);
    ~PhoneApplet();

    // The paintInterface procedure paints the applet to screen
    void paintInterface(QPainter *painter,
            const QStyleOptionGraphicsItem *option,
            const QRect& contentsRect);
    void init();
    void proc_click();
    void proc_ring();
    void play_ring( QString, QString );
    int proc_dlecommand( char* );
    MyModem* Modem() { return m_modem; }
    int volume() { return m_iVolume; }
    void abortConnectThreads();
    void abortDial();

    void setVolume( int );
    QString modemInits() { return m_strModemInits; }
    bool resetAfterHangup() { return m_bWantReset; }
    bool keepModemOpen() { return m_bWantKeep; }
    void closeInfoBox();

public slots:
    void dataUpdated( const QString&, const Plasma::DataEngine::Data& );

public:
    QString m_strModemFilename;
    QString m_strNewModemFilename;

private:
    void paintInterfaceState(QPainter *painter,
            const QStyleOptionGraphicsItem *option,
            const QRect& contentsRect,
            Plasma::Svg&);
    // functions from xringd.c
    //int modem_open(char *file, bool readonly );
    //bool modem_talk( char* cmd, char* resp );   // for member file number
    void readConfig();
    void saveConfig();
    void modem_close(); // for member file number
    void set_state( eMODEM_STATE newstate );
    void setVolumeIcon();
    void setConnectIcon();
    //int modem_volume( int, int );
    void setBellIcon();

    void MessageBox(const QString, bool error=TRUE);
    void ErrMessage(int);
    void mousePressEvent(QGraphicsSceneMouseEvent *);
    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    virtual QList< QAction * > contextualActions();
    void AbortAllThreads();
    void digitClicked(char digit);
    void quickClicked( int ix, bool pn = FALSE );
    bool wantNewModemBox();
    bool initAreYouSureBox();
    QString historyHead();
    void historyAdd( QString );
    bool isQuick( int* digit, bool* numbfollows);
    QString quickNumber();
    QString quickNumber( QString );
    QString quickNumber( int );
    QString quickData();

    virtual void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
    virtual void dragMoveEvent(QGraphicsSceneDragDropEvent *event);
    virtual void dropEvent( QGraphicsSceneDragDropEvent *event );
    virtual void createConfigurationInterface(KConfigDialog *parent);
    void newAppletSetup();
    void newQuickListSetup();
    QString faxFileDialog();
    void resetAfterFax();
    void startEngine();
    void addQuickContacts();
    void removeQuickContacts();

private slots:
    void digitClicked1()    { digitClicked( '1' ); }
    void digitClicked2()    { digitClicked( '2' ); }
    void digitClicked3()    { digitClicked( '3' ); }
    void digitClicked4()    { digitClicked( '4' ); }
    void digitClicked5()    { digitClicked( '5' ); }
    void digitClicked6()    { digitClicked( '6' ); }
    void digitClicked7()    { digitClicked( '7' ); }
    void digitClicked8()    { digitClicked( '8' ); }
    void digitClicked9()    { digitClicked( '9' ); }
    void digitClicked0()    { digitClicked( '0' ); }
    void digitClickedAst()  { digitClicked( '*' ); }
    void digitClickedPnd()  { digitClicked( '#' ); }
    void digitClickedFlsh();// { digitClicked( '!' ); }
    void quickClicked0()    { quickClicked( 0 ); }
    void quickClicked1()    { quickClicked( 1 ); }
    void quickClicked2()    { quickClicked( 2 ); }
    void quickClicked3()    { quickClicked( 3 ); }
    void quickClicked4()    { quickClicked( 4 ); }
    void quickClicked5()    { quickClicked( 5 ); }
    void quickClicked6()    { quickClicked( 6 ); }
    void quickClicked7()    { quickClicked( 7 ); }
    void quickClicked8()    { quickClicked( 8 ); }
    void quickClicked9()    { quickClicked( 9 ); }
    void quickPaneClicked0()    { quickClicked( 0, TRUE ); }
    void quickPaneClicked1()    { quickClicked( 1, TRUE ); }
    void quickPaneClicked2()    { quickClicked( 2, TRUE ); }
    void quickPaneClicked3()    { quickClicked( 3, TRUE ); }
    void quickPaneClicked4()    { quickClicked( 4, TRUE ); }
    void quickPaneClicked5()    { quickClicked( 5, TRUE ); }
    void quickPaneClicked6()    { quickClicked( 6, TRUE ); }
    void quickPaneClicked7()    { quickClicked( 7, TRUE ); }
    void quickPaneClicked8()    { quickClicked( 8, TRUE ); }
    void quickPaneClicked9()    { quickClicked( 9, TRUE ); }
    void deleteText();
    void AnswerCall();
    void timeElapsed();
    void DialCall();
    void RedialCall();
    void doDial( QString );
    void doFax();
    void faxSendDone();
    void faxRecvDone();
    void cancelFax();
    void toggle_sound();
    void clickVolume();
    void blinkit();
    //void unclick() { m_bMousePress = FALSE; update(); }
    void proc_endringseq();
    void toggle_daemon();
    void setDaemonIcon();
    void StartDaemon();
    void AbortDaemon();
    void HangUp( bool error = FALSE, bool fromdial = FALSE );
    void CallHungUp();
    void UIButtonClick();
    void CallAnswered();
    void PostConnectEnd();
    void ToggleGUI();
    void InfoBox();
    void resetConfigDialog();
    void saveConfigDialog();
    void endConfigDialog();
    void soundFileDialog();
    void modemFileDialog();
    void modemInitsBox();
    void modemResetBox();
    void sendFaxCmdBox();
    void recvDialFaxCmdBox();
    void recvPickupFaxCmdBox();
    void convertToFaxCmdBox();
    void faxPrefixBox();
    void faxViewerBox();
    void contactsBox();
    //void keyShortcutBox();
    void test_ring();
    void clickNext();
    void clickPrev();
    void historySelect( int );
    void historyClick( const QModelIndex& );
    void startHistoryList();
    void startQuickList( int );
    void quickListClicked( const QModelIndex& );
    void quickListSelect(int);
    void contactsSelect( int );
    void contactsClick( const QModelIndex& );
    void startContacts();
    void dataDone();
    void doTextEdited( QString );
    void doTextChanged( QString );
    void useSearch();
    void cancelSearch();
    void contactsOps();

private:
    KIcon m_iconMain;
    KIcon m_iconRing;
    KIcon m_iconRing2;
    KIcon m_iconConnect;
    KIcon m_iconConnect2;
    KIcon m_iconDial;
    KIcon m_iconDial2;
    KIcon m_iconTalk;
    KIcon m_iconTalk2;
    KIcon m_iconHangup;
    KIcon m_iconHangup2;
    KIcon m_iconNogood;
    KIcon m_iconNogood2;
    KIcon m_iconSendFax;
    KIcon m_iconSendFax2;
    KIcon m_iconRecvFax;
    KIcon m_iconRecvFax2;

    eMODEM_STATE m_eState;
    bool m_bWantSound;
    bool m_bWantReset;
    bool m_bWantKeep;
    bool m_bWantFaxOutput;
    bool m_bAlternate;
    int m_iVolume;
    bool m_bVolumeRising;
    bool m_bMouseOver;
    //bool m_bMousePress;
    bool m_bAkonadiHasData;
    bool m_bConfigChanged;

    //struct timeval m_tmBef;

    QTimer* m_QTimerBlink;
    QTimer* m_QTimerEndRings;
    QTimer* m_QTimerSeconds;
    QTimer* m_QTimerMouse;
    QTimer* m_QTimerContacts;

    RingDaemonThread* m_DaemonThread;
    ModemAnswerThread* m_AnswerThread;
    ModemHangupThread* m_HangupThread;
    ModemDialThread* m_DialThread;
    PostConnectThread* m_PostConnectThread;
    SendFaxThread* m_SendFaxThread;
    RecvFaxThread* m_RecvFaxThread;

    MyPhoneNumber* m_PhoneNumber;
    MyAppletSetup* m_AppletSetup;
    MyFaxSetup* m_FaxSetup;
    InfoBoxes* m_InfoBox;
    QTabWidget* m_QuickSetup;

    //QString m_prevNumber; -- USE callMade
    QStringList m_quickDial;
    QList<MyQuickSetup*> m_QuickPanes;
    QList<QLineEdit*> m_quickEdit;
    QList<QToolButton*> m_quickButton;
    KHistoryComboBox* m_callsMade;
    MyListWidget* m_callsList;
    ContactsListWidget* m_contactsList;

    int m_iInHistory;
    int m_iInQuick;
    int m_iInContacts;
    eCONTACTS_STATE m_eContactsData;
    bool m_bWantEngine;
    bool m_bWantSearch;
    bool m_bMergeQuick;
    bool m_bUseSearch;

    QProcess m_playSound;
    MyModem* m_modem;

    KConfigGroup propConfig;
    KConfigGroup optConfig;
    KConfigGroup callConfig;

    // program configuration copied to these
    QString m_strModemInits;
    QString m_strPlaySound;
    QString m_strSoundFile;
    QString m_strFaxSendCmd;
    QString m_strFaxRecvCmd;
    QString m_strFaxPickupCmd;
    QString m_strFileToFaxCmd;
    QString m_strFaxViewCmd;
    QString m_strFaxPrefix;

    Plasma::DataEngine* m_Contacts;
};


// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(phoneapplet, PhoneApplet)
#endif
