/* 
 * File:   phoneapplet_contacts.h
 * Author: david
 *
 * Created on February 18, 2009, 3:09 PM
 */

#ifndef _PHONEAPPLET_CONTACTS_H
#define	_PHONEAPPLET_CONTACTS_H

#include "phoneapplet.h"


class MyListWidget : public QListWidget {
public:
    MyListWidget() {
        // attributes
        setPalette(QPalette(Qt::white));
        setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    }

    void setRow( int row ) {
        setCurrentRow( row );
        scrollToItem( currentItem() );
    }

    int prevRow() {
        int cnt = count();
        if (cnt < 1)
            return -1;

        int row = currentRow();
        --row;
        if (row < 0)
            row = cnt - 1;
        setRow( row );
        return row;
    }

    int nextRow() {
        int cnt = count();
        if (cnt < 1)
            return -1;

        int row = currentRow();
        ++row;
        if (row >= cnt)
            row = 0;
        setRow( row );
        return row;
    }

    virtual QString currNumber()        { return numberOfEntry( currentItem()->text() ); }
    virtual QString currName()          { return nameOfEntry( currentItem()->text() ); }
    virtual QString numberAt( int ix )  { return numberOfEntry( item( ix )->text() ); }
    virtual QString nameAt( int ix )    { return nameOfEntry( item( ix )->text() ); }

    static QString numberOfEntry( QString entry ) {
        QStringList pieces = entry.split( ':' );
        int Count = pieces.count();
        if  ( Count < 2 )
            return entry;
        for ( int n = 0; n < Count; ++n ) {
            // first numerical entry?
            if  ( pieces[ n ].at( 0 ).isDigit() )
                return pieces[ n ];
        }
        return entry;
    }

    static QString nameOfEntry( QString entry ) {
        QStringList pieces = entry.split( ':' );
        int Count = pieces.count();
        if  ( Count < 2 )
            return entry;
        for ( int n = 0; n < Count; ++n ) {
            // first non-numerical entry?
            if  ( !pieces[ n ].at( 0 ).isDigit() )
                return pieces[ n ];
        }
        return entry;

    }
};

class ContactsListWidget : public MyListWidget {
public:
    ContactsListWidget() {
        // additional attributes
        setSortingEnabled(TRUE);
        QFont font = QListWidget::font();
        font.setPointSize(6);
        setFont(font);
    }

    QString textAt( int ix )    { return item( ix )->text(); }

    QString numberAt(int ix) {
        QString contact = textAt(ix);
        QStringList contactdata = contact.split(':');
        if (contactdata.count() < 1)
            return "";
        return contactdata[ contactdata.count() - 1 ];
    }

    QString searchFor(QString text ) {
        // do we need another search?
        if ( currentItem()->text().indexOf(text, 0, Qt::CaseInsensitive) != 0)
            // now do it
            keyboardSearch(text);
        // let's see what we got
        return currentItem()->text();
    }

    QString richSearchFor(QString text ) {
        QString contact = searchFor(text);
        if (contact.length() < 1)
            return "";
        if (contact.indexOf(text, 0, Qt::CaseInsensitive) == 0)
            return richItem( contact, text );
        return "";
    }

    QString richItem(QString item, QString hiLight = "") {
        QString rich = "<html><font color=blue>";
        
        int len = hiLight.length();
        if (len > 0) {
            rich += "<i><b>";
            rich += item.left(len);
            rich += "</b></i>";
        }
        rich += item.mid(len);
        rich.replace( "\n", "<br>");
        rich += "</font></html>";
        return rich;
    }

    QString lineItem( int ix ) {
        QString text = item( ix )->text();
        text.replace( '\n', ' ' );
        return text;
    }

    void addContact( const Plasma::DataEngine::Data& data ) {
        QString entry;
        QString prefix;

        QStringList keys = data.uniqueKeys();
        int nKeys = keys.count();

        for (int n = 0; n < nKeys; ++n) {
            if (keys[n].contains(ONE_CONTACT_PHONE, Qt::CaseInsensitive)) {
                entry = entryFromHash( data, keys[n], &prefix );
                if  ( entry.length() < 1 )
                    continue;
                // No duplicates but allow update
                // coding this way no very economica
                QList<QListWidgetItem*> found = findItems(prefix, Qt::MatchStartsWith);
                if (found.count() > 0) {
                    // replace, so first remove it
                    int ix = row(found[0]);
                    setCurrentRow(ix);
                    delete takeItem(ix);
                }
                // add
                addItem(entry);
            }
        }
    }

    void addQuickContact( int ix, QString quick ) {
        addContact( hashFromQuick( ix, quick ) );
    }

    void delQuickContact(int ix, QString quick) {
        QString prefix;
        QString entry = entryFromHash( hashFromQuick(ix, quick), quickKey( ix ), &prefix );
        QList<QListWidgetItem*> found = findItems(entry, Qt::MatchFixedString);
        if (found.count() > 0) {
            // remove it
            int ix = row(found[0]);
            //setCurrentRow(ix);
            delete takeItem(ix);
        }
    }

private:
    QString entryFromHash(Plasma::DataEngine::Data data, QString key, QString* prefix ) {
        static QString entry;
        QString phone;
        QString number;

        number = data.value( key ).toString();
        number.remove(':');
        number = number.trimmed();
        if (number.length() < 1)
            return "";

        // assuming all keys like phone-home, phone-fax or such, no plain "phone"
        phone = key;
        phone.remove(ONE_CONTACT_PHONE);
        phone.remove('-');
        phone.remove(':');
        phone = phone.trimmed();
        if (phone.length() < 1)
            return "";

        entry = data.value(ONE_CONTACT_NAME).toString().simplified();
        entry.remove(':');
        entry += '\n';
        entry += phone;
        entry += ':'; // this be syntax not found elesewhere in this data!!

        *prefix = entry;

        return entry + number;
    }

    Plasma::DataEngine::Data hashFromQuick( int ix, QString quick ) {
        Plasma::DataEngine::Data hash;
        QString name;
        QString number;

        // split based on colon like above
        int colon = quick.indexOf(':');
        // item with digits will be a phone number, rest will be name
        // if NO colon, Name = QUICK#ix
        if  ( colon < 0 ) {
            name = "Quick#";
            QTextStream( &name ) << ix;
            number = quick;
        }
        else if ( colon == 0 || colon >= quick.length() ) {
            // no way to parse so ...
            if  ( quick.at( 0 ).isDigit() ) {
                name = "Quick#";
                QTextStream( &name ) << ix;
                number = quick;
            }
            else {
                return hash;        // empty
            }
        }
        else {
            QString item1 = quick.left( colon ).trimmed();
            QString item2 = quick.mid( colon + 1 ).trimmed();
            if  ( item2.at( 0 ).isDigit() ) {
                number = item2;
                name = item1;
            }
            else if ( item1.at( 0 ).isDigit() ) {
                number = item1;
                name = item2;
            }
            else {
                return hash;        // empty
            }
        }

        // if I got here, I have data
        QString phoneKey = quickKey( ix );
        hash[ ONE_CONTACT_NAME ] = name;
        hash[ phoneKey ] = number;
        return hash;
    }
    
    QString quickKey( int ix ) {
        QString phoneKey;
        QTextStream( &phoneKey ) << ONE_CONTACT_PHONE << "-#" << ix;
        return phoneKey;
    }
};



#endif	/* _PHONEAPPLET_CONTACTS_H */

