/* 
 * File:   phoneapplet_dialogs.h
 * Author: david
 * Java style mixed declarations but ...
 *
 * Created on January 5, 2009, 11:17 AM
 */

#ifndef _PHONEAPPLET_DIALOGS_H
#define	_PHONEAPPLET_DIALOGS_H

class MyPhoneNumber : public QDialog {
public:
    MyPhoneNumber(PhoneApplet* parent) {
        KIconLoader kil;

        m_Parent = parent; // Caller, simplest UI:
        UI.setupUi( this );

        setWindowIcon( KIcon( "internet-telephony" ) );
        UI.toolButton_Close->setIcon( KIcon( "system-shutdown" ));
        UI.toolButton_Bell->setIcon( KIcon( "preferences-desktop-notification-bell" ) );
        BellPath = kil.iconPath( "preferences-desktop-notification-bell", KIconLoader::Toolbar );
        UI.toolButton_Volume->setIcon( KIcon( "audio-volume-high"));
        VolumePath = kil.iconPath( "audio-volume-high", KIconLoader::Toolbar );
        ConfigPath = kil.iconPath( "preferences-system", KIconLoader::Toolbar );
        UI.toolButton_Info->setIcon( KIcon( "help-about" ));
        UI.toolButton_Flash->setIcon( KIcon( "media-record"));
        FlashPath = kil.iconPath( "media-record", KIconLoader::Toolbar );
        UI.toolButton_Redial->setIcon( KIcon( "mark_as_spam" ));
        RedialPath = kil.iconPath( "mark_as_spam", KIconLoader::Toolbar );
        UI.toolButton_Config->setIcon( KIcon( "preferences-system" ));
        UI.toolButton_Up->setIcon( KIcon( "arrow-up" ) );
        UpPath = kil.iconPath( "arrow-up", KIconLoader::Toolbar );
        UI.toolButton_Down->setIcon( KIcon( "arrow-down" ) );
        DownPath = kil.iconPath( "arrow-down", KIconLoader::Toolbar );
        UI.toolButton_Contacts->setIcon( KIcon( "face-smile" ) );
        ContactsPath = kil.iconPath( "face-smile", KIconLoader::Toolbar );
        UI.toolButton_Fax->setIcon( KIcon( "document-print-preview" ) );
        FaxPath = kil.iconPath( "document-print-preview", KIconLoader::Toolbar );

        adoptedWidget = NULL;
    }

    ~MyPhoneNumber() {
        unadoptWidget();    // do not want to delete that here!!
    }

    virtual void setVisible( bool vis ) {
        QDialog::setVisible( vis );
        if  ( !vis )
            m_Parent->closeInfoBox();
    }

    QString number() { return UI.lineEdit_Number->text().trimmed(); }
    void setNumber( QString number ) { UI.lineEdit_Number->setText( number.trimmed() ); }
    void setPrefix( QString pref ) { m_Prefix = pref; }

    void setStatusLine(QString txt) {
        QFont font;
        font.setPointSize(6);
        font.setBold(false);
        font.setWeight(50);
        UI.textEdit_Status->setFont(font);
        UI.textEdit_Status->setText(txt);
    }

    void clearStatusLine() { UI.textEdit_Status->setText( "" ); }

    void resetTimeElapsed() { 
        m_secs = 0;
        m_Prefix = "";
        setTimeElapsed();
    }

    void setTimeElapsed() {
        QFont font;
        font.setPointSize(8);
        font.setBold(false);
        font.setWeight(50);
        UI.textEdit_Status->setFont(font);

        int sec = m_secs++;
        QString txt= "<html><font color=green>";
        QString tim;
        if  ( sec >= 3600 )
           
            tim.sprintf( "%02d:%02d:%02d", sec / 3600, (sec % 3600) / 60, sec % 60 );
        else
            tim.sprintf( "%02d:%02d", sec / 60, sec % 60 );
        txt += m_Prefix;
        txt += tim;
        txt += "</font></html>";
        UI.textEdit_Status->setText(txt);
    }

    void adoptWidget( QWidget* widget ) {
        if  ( adoptedWidget != NULL )
            return; //just in case

        adoptedWidget = widget;
        formerParent = widget->parentWidget();
        widget->setParent( this );

        widgetGeometry = UI.lineEdit_Number->geometry();
        QRect toAdd = UI.textEdit_Status->geometry();
        widgetGeometry.setHeight( widgetGeometry.height() + toAdd.height() );

        UI.lineEdit_Number->setVisible( FALSE );
        UI.textEdit_Status->setVisible( FALSE );

        widget->setGeometry( widgetGeometry );
        widget->setVisible(TRUE);
        widget->setFocus();
    }

    bool unadoptWidget() {
        if  ( adoptedWidget == NULL )
            return FALSE;

        adoptedWidget->setParent( formerParent );
        adoptedWidget->setVisible(FALSE);

        UI.lineEdit_Number->setVisible( TRUE );
        UI.textEdit_Status->setVisible( TRUE );

        adoptedWidget = NULL;

        UI.lineEdit_Number->setFocus();
        return TRUE;
    }

    bool isAdopted() { return adoptedWidget != NULL; }
    QWidget* adopted() { return adoptedWidget; }

public:
    Ui::PhoneDialog UI;

    QString FlashPath;
    QString RedialPath;
    QString BellPath;
    QString VolumePath;
    QString ConfigPath;
    QString UpPath;
    QString DownPath;
    QString ContactsPath;
    QString FaxPath;

    QString m_Prefix;

private:
    PhoneApplet* m_Parent;
    int m_secs;
    QWidget* adoptedWidget;
    QWidget* formerParent;
    QRect widgetGeometry;
};

class MyAppletSetup : public QWidget {
public:
    MyAppletSetup() {
        KIconLoader kil;

        UI.setupUi( this );

        UI.toolButton_Modem->setIcon( KIcon( "hardware" ) );
        UI.toolButton_Init->setIcon( KIcon( "applications-science" ) );
        UI.toolButton_Player->setIcon( KIcon( "media-playback-start" ) );
        UI.toolButton_Sound->setIcon( KIcon( "folder-sound" ) );
        UI.toolButton_ModemReset->setIcon( KIcon( "media-seek-forward" ) );
        UI.checkBox_ModemReset->setIcon( KIcon( "system-restart" ) );
        resetPath = kil.iconPath( "system-restart", KIconLoader::Toolbar );
        UI.checkBox_ModemKeep->setIcon( KIcon( "system-lock-screen" ) );
        keepPath = kil.iconPath( "system-lock-screen", KIconLoader::Toolbar );
        UI.toolButton_Contacts->setIcon( KIcon( "face-smile") );
        searchPath = kil.iconPath( "page-zoom", KIconLoader::Toolbar );
        UI.checkBox_ContactsSearch->setIcon( KIcon( "page-zoom") );
        enginePath = kil.iconPath( "akonadi", KIconLoader::Toolbar );
        UI.checkBox_Akonadi->setIcon( KIcon( "akonadi") );
    }

    ~MyAppletSetup() {}

public:
    Ui::SetupWidget UI;
    QString keepPath;
    QString resetPath;
    QString enginePath;
    QString searchPath;
};

class MyFaxSetup : public QWidget {
public:
    MyFaxSetup() {
        KIconLoader kil;

        UI.setupUi( this );

        UI.toolButton_SendFax->setIcon( KIcon("media-seek-forward" ) );
        UI.toolButton_Convert->setIcon( KIcon("system-switch-user"));
        UI.toolButton_RecvFax->setIcon( KIcon( "media-seek-backward"));
        UI.toolButton_PickupFax->setIcon( KIcon( "media-skip-backward"));
        UI.toolButton_ViewFax->setIcon( KIcon( "document-preview" ) );
        UI.toolButton_ShowOutput->setIcon( KIcon( "edit-select-all" ));
        UI.toolButton_Prefix->setIcon( KIcon( "folder-print" ));
    }

    ~MyFaxSetup() {}

public:
    Ui::FaxSetupWidget UI;
};

class MyQuickSetup : public QWidget {
public:
    MyQuickSetup() {
        UI.setupUi( this );
    }

public:
    Ui::QuickWidget UI;
};

// some useful shortcuts
static QString Message( int err_no ) {
    return ( err_no >= 0 )
            ? MESSAGES[ 0 ]
            : ( err_no < MXERR )
                ? MESSAGES[ 0 ]
                : MESSAGES[ -err_no ];
}

static QString HTMLImg( QString path ) {
    QString img = "<img src=\"";
    img += path;
    img += "\" height=16 width=16>";
    return img;
}

class InfoBoxes : public QMessageBox {
public:
    InfoBoxes(PhoneApplet* parent):QMessageBox() {
        m_Parent = parent;
    }
    
    void setPhoneNumberDlg( MyPhoneNumber* pn )  { m_PhoneNumber = pn; }
    void setAppletSetupDlg( MyAppletSetup* su )  { m_AppletSetup = su; }
    void setQuickSetupDlg( QTabWidget* qs )    { m_QuickSetup = qs; }
    void setFaxSetupDlg( MyFaxSetup* fs )        { m_FaxSetup = fs; }

    void InfoBox() {
        QString info;
        info = "<html><center>";
        info += m_Parent->name();
        info += " &mdash;0.90-rcp<p><b>";
        info += tr("Speaker-Phone Control Plasma-Applet");
        info += "</b><p>";
        info += tr("by David Baron");
        info += "<hr>";
        info += tr(
                "Click applet to answer phone when ringing, to hang up, and to toggle the phone dialog when idle.");
        info += "<p>";
        info += tr("Click OK on the phone as you would on a real one.");
        info += "<br>";
        info += HTMLImg(m_PhoneNumber->FlashPath);
        info += tr(" pick up/flash ");
        info += HTMLImg(m_PhoneNumber->RedialPath);
        info += tr(" redial number");
        info += "<br>";
        info += HTMLImg(m_PhoneNumber->VolumePath);
        info += tr(" set volume ");
        info += HTMLImg(m_PhoneNumber->BellPath);
        info += tr(" toggle ring tone ");
        info += HTMLImg(m_PhoneNumber->ConfigPath);
        info += tr(" configure");
        info += "<br>";
        info += tr("Quick-dial #0-#9 number ");
        info += HTMLImg(m_PhoneNumber->UpPath);
        info += tr(" list ");
        info += HTMLImg(m_PhoneNumber->DownPath);
        info += tr(" save");
        info += "<br>";
        info += HTMLImg(m_PhoneNumber->ContactsPath);
        info += tr(" contacts ");
        info += HTMLImg(m_PhoneNumber->UpPath);
        info += HTMLImg(m_PhoneNumber->DownPath);
        info += tr(" browse");
        info += "<br>";
        info += tr("Contacts search ");
        info += HTMLImg(m_PhoneNumber->UpPath);
        info += tr(" list ");
        info += HTMLImg(m_PhoneNumber->DownPath);
        info += tr(" next");
        info += "<br>";
        info += HTMLImg(m_PhoneNumber->UpPath);
        info += HTMLImg(m_PhoneNumber->DownPath);
        info += tr(" call history");
        info += "<br>";
        info += HTMLImg(m_PhoneNumber->FaxPath);
        info += tr(" fax");
        info += "<p>";
        info += tr("Please notify me of any problems ");
        info += "<a href=mailto:d_baron@012.net.il>d_baron@012.net.il</a><hr>";
        info += "<i>";
        info += tr("Enjoy!");
        info += "</i></center></html>";

        setText(info);
        setVisible(TRUE);
    }

    bool wantNewModemBox() {
        QString info;
        info = "<html><font color=green><i>";
        info += m_Parent->m_strModemFilename;
        info += "</i>";
        info += tr("is working now.");
        info += "</font><p><font color=red><i>";
        info += m_Parent->m_strNewModemFilename;
        info += "</i>";
        info += tr("will be used when applet restarts.");
        info += "<br><b>";
        info += tr("Are you sure?");
        info += "</b></font></html>";

        return QMessageBox::warning(m_AppletSetup, m_Parent->name(), info,
                QMessageBox::Yes | QMessageBox::No)
                == QMessageBox::Yes;
    }

    bool initAreYouSureBox() {
        QString info;
        info = "<html><b>";
        info += tr("You have changed the modem initialization commands!");
        info += "</b><p><font color=red><b>";
        info += tr("Are you sure?");
        info += "</b>";

        return QMessageBox::warning(m_AppletSetup, m_Parent->name(), info,
                QMessageBox::Yes | QMessageBox::No)
                == QMessageBox::Yes;
    }

    void modemInitsBox() {
        QString info = "<html><center><b>";
        info += tr("Modem Initialization");
        info += "</b></center><p>";
        info += tr(
                "These are for Hayes compatable Rockwell chip (336, etc) speakerphone modems but should work with others as well.");
        info += "<p><font color=green><b>";
        info += m_AppletSetup->UI.lineEdit_Init->text();
        info += "</b></font><p>";
        info += "<ul><li>";
        info += tr("S7=# remote carrier wait secs");
        info += "<li>";
        info += tr("S6=# dialtone wait secs");
        info += "<li>";
        info += tr("S0=# rings before auto-answer&mdash;0 no answer.");
        info += "<br><i>";
        info += tr("If programs are running which need auto-answer, omit here.");
        info += "</i><li>";
        info += tr("M# speaker&mdash;0 off, 1 till answer, 2 always on, 3 during answer");
        info += "<li>";
        info += tr("V# 0 numerical responses, 1 textual responses");
        info += "<li>";
        info += tr("X# 4 all modem responses");
        info += "<li>&amp;";
        info += tr("c1 DCD follows carrier");
        info += "&mdash;<i>";
        info += tr("Best do not change");
        info += "</i><li>";
        info += tr("E0 echo 0 off, 1 on&mdash;<i>Best do not change!");
        info += "</i><li>";
        info += tr("Q0 DTR set");
        info += "&mdash;<i>";
        info += tr("Best do not change!");
        info += "</i></ul><p><center><font color=red><i>";
        info += tr("Be carefull in changing any of these!");
        info += "</i></font></center></html>";

        setText(info);
        setVisible(TRUE);
    }

    void modemResetBox() {
        QString info = "<html><center><b>";
        info += tr("Modem Access Options.");
        info += "</b></center><p>";
        info += "<font color=red>";
        info += tr("Best choices depend on the modem and other programs that access it.");
        info += "</font><p>";
        info += HTMLImg(m_AppletSetup->keepPath);
        info += tr(" Keep modem device access between sessions");
        info += "&mdash;";
        info += tr("connect times will be quicker but may effect other programs that access modem.");
        info += "<br>";
        info += HTMLImg(m_AppletSetup->resetPath);
        info += tr("Always reset modem on release");
        info += "&mdash;";
        info += tr("desirability depends on hardware. Not resetting yields quicker disconnect.");

        setText(info);
        setVisible(TRUE);
    }

    void contactsBox() {
        QString info = "<html><center><b>";
        info += tr("Contacts Options.");
        info += "</b></center><p>";
        info += HTMLImg(m_AppletSetup->enginePath);
        info += tr(" Use Akonadi dataengine to access KDEPIM contacts");
        info += "<br><font color=red>";
        info += tr("Akonadi must be set up for KAddressbook contacts and its Plasma dataengine installed.");
        info += "</font><br><i>";
        info += tr("Dataengine will work now. If you uncheck, it will not be used on the next login.");
        info += "</i><p>";
        info += HTMLImg(m_AppletSetup->searchPath);
        info += tr(" Live contact search");
        info += "&mdash;";
        info += tr("will search contacts as you type the name.");
        info += "<p><b>#</b> ";
        info += tr("Merge quick-dial numbers with contacts search.");
        info += "<br>";

        setText(info);
        setVisible(TRUE);
    }

    void sendFaxCmdBox() {
        QString info = "<html><center><b>";
        info += tr("Command to Send Fax:");
        info += "</b></center><p>";
        info += "<font color=green><b>";
        info += m_FaxSetup->UI.lineEdit_SendFax->text();
        info += "</b></font><p><font color=green>";
        info += CMDARG1;
        info += "</font>&mdash;";
        info += tr("Modem Device (");
        info += m_Parent->m_strModemFilename;
        info += ")<br><font color=green>";
        info += CMDARG2;
        info += "</font>&mdash;";
        info += tr("Fax telephone number");
        info += "<br><font color=green>";
        info += CMDARG3;
        info += "</font>&mdash;";
        info += tr("Files to send");
        info += "<p><font color=red>";
        info += tr("Command line must support *-wildcard.");
        info += "</font><p>";

        info.replace(CMDARG1, HTMLARG1);
        info.replace(CMDARG2, HTMLARG2);
        info.replace(CMDARG3, HTMLARG3);

        setText(info);
        setVisible(TRUE);
    }

    void recvFaxCmdBox(bool dial) {
        QString info = "<html><center><b>";
        info += tr("Command to Receive Fax:");
        info += "</b></center><p>";
        info += "<font color=green><b>";
        info += dial ? m_FaxSetup->UI.lineEdit_RecvFax->text()
                : m_FaxSetup->UI.lineEdit_PickupFax->text();
        info += "</b></font><p><font color=green>";
        info += CMDARG1;
        info += "</font>&mdash;";
        info += tr("Modem Device (");
        info += m_Parent->m_strModemFilename;
        info += ")<br><font color=red>";
        info += tr("Flag if call has been answered in phoneapplet: -w for efax.");
        info += "</font><br><font color=green>";
        info += CMDARG3;
        info += "</font>&mdash;";
        info += tr("File name");

        info.replace(CMDARG1, HTMLARG1);
        //info.replace( CMDARG2, HTMLARG2 );
        info.replace(CMDARG3, HTMLARG3);

        setText(info);
        setVisible(TRUE);
    }

    void convertToFaxCmdBox() {
        QString info = "<html><center><b>";
        info += tr("Command for file conversion.");
        info += "</b></center><p>";
        info += "<font color=green><b>";
        info += m_FaxSetup->UI.lineEdit_Convert->text();
        info += "</b></font><p><font color=green>";
        info += CMDARG1;
        info += "</font>&mdash;";
        info += tr("file to convert (");
        info += m_Parent->m_strModemFilename;
        info += ")<br><font color=green>";
        info += CMDARG2;
        info += "</font>&mdash;";
        info += tr("converted file");

        info.replace(CMDARG1, HTMLARG1);
        info.replace(CMDARG2, HTMLARG2);
        info.replace(CMDARG3, HTMLARG3);

        setText(info);
        setVisible(TRUE);
    }

    void faxPrefixBox() {
        QString info = "<html><center><b>";
        info += tr("Fax File Name Prefix");
        info += "</b></center><p>";
        info += "<font color=green><b>";
        info += m_FaxSetup->UI.lineEdit_Prefix->text();
        info += "</b></font><p>";
        info +=
    tr("Received faxes will be have file names beginning with this pattern which can include a directory path.");
        setText(info);
        setVisible(TRUE);
    }

    void faxViewerBox() {
        QString info = "<html><center><b>";
        info += tr("Fax Viewer Program");
        info += "</b></center><p>";
        info += "<font color=green><b>";
        info += m_FaxSetup->UI.lineEdit_ViewFax->text();
        info += "</b></font><p>";
        info +=
    tr("Choice include \"gwenview\" which is part of KDE, \"viewfax\" which is included in most distros, and others.");
        setText(info);
        setVisible(TRUE);
    }

    /*void keyShortcutBox() {
        QString info = "<html><center><b>Global Keyboard Shortcut.</b></center><p>";
        info += "Key-sequence to show or hide the call dialog.<br>";
        info += "Click widget and then key the desired shortcut, [x} to remove.<p></html>";

        setText(info);
        setVisible(TRUE);
    }*/

private:
    PhoneApplet *m_Parent;
    MyPhoneNumber* m_PhoneNumber;
    MyAppletSetup* m_AppletSetup;
    MyFaxSetup* m_FaxSetup;
    QTabWidget* m_QuickSetup;
};

#endif	/* _PHONEAPPLET_DIALOGS_H */

