/* 
 * File:   phoneapplet_threads.h
 * Author: david
 * Java style mixed declarations but ...
 *
 * Created on January 5, 2009, 11:14 AM
 */

#ifndef _PHONEAPPLET_THREADS_H
#define	_PHONEAPPLET_THREADS_H

#include "phoneapplet.h"
#include "mymodem.h"


//static sigset_t sig_mask;

static QMutex queuemutex;

// parent of all the worker threads below:
class ModemOpThread : public QThread {
public:
    ModemOpThread(PhoneApplet* parent) : QThread(parent) {
        m_Parent = parent;
        m_modem = m_Parent->Modem();
        setTerminationEnabled( TRUE );
        init();
    }
    ~ModemOpThread() {}

    bool abort() {
        if  ( !isRunning() )
            return FALSE;

        m_bAbort = TRUE;
        wait();
        return TRUE;
    }

    bool result() { return m_iRet == 0; }
    bool aborted() { return m_bAbort; }
    int error() { return m_iRet; }

protected:
    void init() {
        m_bAbort = FALSE;
        m_iRet = 0;
    }

    PhoneApplet* m_Parent;
    int m_iRet;
    bool m_bAbort;
    MyModem* m_modem;
};

class RingDaemonThread : public ModemOpThread {
public:
    RingDaemonThread(PhoneApplet* parent = 0) : ModemOpThread(parent) {
    /*    struct sigaction sa;

        ::sigemptyset(&sig_mask);
	::sigaddset(&sig_mask, SIGUSR2);
	sa.sa_mask = sig_mask;
        sa.sa_handler = dummy;
        sa.sa_flags = 0;
	::sigaction(SIGUSR2, &sa, NULL);
        ::sigset( SIGUSR2, dummy );
    */
    }

    ~RingDaemonThread() {}

    void run() {
        init();

        m_DaemonModem.setFilename(m_strModemFilename);
        m_fn = m_DaemonModem.open(TRUE); /* first open */

        // this should be tested beforehand!
        if (m_fn < 0) {
            m_iRet = ERRNO_NOMODEM;
            return;
        }

        // code from xringd.c

        int c;
        if (ioctl(m_fn, TIOCGICOUNT, &c) != 0) {
            m_iRet = ERRNO_BADLINUX;
            return;
        }

        long msec;
        struct timeval tmBef, tm;

        gettimeofday(&tmBef, NULL);

        /* main daemon loop */
        m_bAbort = FALSE;
        while (!m_bAbort) {
            /* wait on RI line */
            if (ioctl(m_fn, TIOCMIWAIT, TIOCM_RNG) != 0) {
                if  (EINTR == errno) {
                    continue;
                }
                /*
                 * XXX EIO may occur: port hung up by other process!
                 * Reopen it - danger: reopening failing continously
                 */
                m_DaemonModem.close();
                usleep(3 * 1000000); // apparently necessary on repopen
                m_fn = m_DaemonModem.open(TRUE);
                continue;
            }

            if  ( m_bAbort )
                break;

            gettimeofday(&tm, NULL);
            msec = (tm.tv_sec - tmBef.tv_sec) * 1000L +
                    (tm.tv_usec - tmBef.tv_usec) / 1000L;
            tmBef = tm; // update the before time!
            if (msec > MSEC_IGNORE_RING)
                m_Parent->proc_ring();
        }

        if (m_fn >= 0)
            m_DaemonModem.close();

        m_iRet = 0;
    }

    void setModemName( QString name ) { m_strModemFilename = name; }
    void pause( long ms , QThread* qt ) { qt->wait( ms ); }
    void pause( QThread* qt ) { qt->wait(); }
    bool result() { return isRunning(); }
    bool abort() {
        bool ret = isRunning();
        if  ( !ret )
            return false;
        // pulse ring
        m_bAbort = true;
        int flags = TIOCM_RNG;
        ioctl( m_fn, TIOCMBIS, &flags );
        usleep( 250000l );
        ioctl( m_fn, TIOCMBIC, &flags );
        usleep( 500000l );
        ioctl( m_fn, TIOCMBIS, &flags );
        usleep( 250000l );
        ioctl( m_fn, TIOCMBIC, &flags );
        usleep( 500000l );
        m_bAbort = false;   //in case this did not work, allow to continue
        return ret;
    }
        // flash ring

private:
    QString m_strModemFilename;
    MyModem m_DaemonModem;
    int m_fn;
};


class ModemAnswerThread : public ModemOpThread {
public:
    ModemAnswerThread( PhoneApplet* parent ):ModemOpThread( parent)  { }
    ~ModemAnswerThread() {
    }

    void run() {
        init();

        m_iRet = m_modem->Init();
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->talk( MODEM_SET_SPEAKERPHONE, MODEM_WAS_OK) != 0)
            m_iRet = ERRNO_SETSPEAKERPHONE;
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->volume( m_Parent->volume()) != 0)
            m_iRet = ERRNO_MODEMVOLUME;
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->talk( MODEM_SET_DTMF, MODEM_WAS_OK) != 0)
            m_iRet = ERRNO_SETDTMF;
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->talk( MODEM_ANSWER, MODEM_VOICE_COM) != 0)
            m_iRet = ERRNO_ANSWERMODEM;
    }
};

class ModemDialThread : public ModemOpThread {
public:

    ModemDialThread(PhoneApplet* parent) : ModemOpThread(parent) { }
    ~ModemDialThread() { }

    void run() {
        init();

        m_iRet = m_modem->Init();
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->talk( MODEM_SET_SPEAKERPHONE, MODEM_WAS_OK) != 0)
            m_iRet = ERRNO_SETSPEAKERPHONE;
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->volume( m_Parent->volume()) != 0)
            m_iRet = ERRNO_MODEMVOLUME;
        if (m_bAbort || m_iRet)
            return;
        if (m_modem->talk( MODEM_SET_DTMF, MODEM_WAS_OK) != 0)
            m_iRet = ERRNO_SETDTMF;
        if (m_bAbort || m_iRet)
            return;

        QString numbercmd;
        numbercmd = MODEM_TONE_DIAL_PRE;
        numbercmd += m_Number;
        numbercmd += MODEM_TONE_DIAL_SUF;
        m_iRet = m_modem->dial( numbercmd.toAscii());
    }

    void setNumber( QString n ) { m_Number = n; }

private:
    QString m_Number;
};

class ModemHangupThread : public ModemOpThread {
public:
    ModemHangupThread( PhoneApplet* parent ):ModemOpThread( parent)  { m_bFromDial = m_bForceReset = FALSE; }
    ~ModemHangupThread() {
    }

    void run() {
        init();

        int ret1 = 0, ret2 = 0;

        if (m_bFromDial)
            m_Parent->abortDial();

        m_Parent->abortConnectThreads();

        m_bAbort = FALSE; // but let this one finish!
        ret1 = m_modem->talk(MODEM_HANGUP, MODEM_WAS_OK, MSEC_RESET_TIMEOUT);
        if (m_bForceReset || (m_Parent->resetAfterHangup() && !m_Parent->keepModemOpen())) {
            ret2 = m_modem->talk(MODEM_RESET, MODEM_WAS_OK, MSEC_RESET_TIMEOUT);
            msleep(MSEC_TWO_SECONDS);
        }

        m_iRet = ret1 != 0
                ? ERRNO_HANGUPMODEM
                : (ret2 != 0
                    ? ERRNO_RESETMODEM
                    : 0);
    }

    void setFromDial ( bool fd ) { m_bFromDial = fd; }
    void setForceReset( bool fd ) { m_bForceReset = fd; }
    bool fromDial() { return m_bFromDial; }
    bool forcedReset()  { return m_bForceReset; }

private:
    bool m_bFromDial;
    bool m_bForceReset;
};

class PostConnectThread : public ModemOpThread {
public:
    PostConnectThread( PhoneApplet* parent ):ModemOpThread( parent)  { }
    ~PostConnectThread() {}

    void add( char digit ) {
        queuemutex.lock();
        m_Queue.enqueue( digit );
        queuemutex.unlock();
    }

    void run() {
        int nRead;

        init();
        m_Queue.clear();

        while ( !m_bAbort  ) {
            m_modem->set_timeout(MSEC_READ_TIMEOUT);

            m_iRet = 0;

            m_modem->lock();
            nRead = m_modem->timed_read();
            m_modem->unlock();

            if (nRead < 0)
                return; // error so simply stop it
            else if (nRead == 0) {
                // no input, do any ququed ops
                runQueue();
                if (m_iRet < 0)
                    return;
            }
            else {
                if (m_modem->testresp(MODEM_DLE)) {
                    m_iRet = m_Parent->proc_dlecommand(m_modem->bytes_read());
                    if (m_iRet < 0)
                        return;
                }

                m_iRet = m_modem->testdialout();
                if (m_iRet < 0)
                    return;
            }
        }
    }

private:
    void runQueue() {
        int ret;

        while ( !m_bAbort && !m_Queue.isEmpty() ) {
            char tone;
            queuemutex.lock();
            tone = m_Queue.dequeue();
            queuemutex.unlock();

            if  ( tone >= 0 && tone < 4 ) {
                ret = m_modem->volume( tone );
                if  ( ret < 0 )
                    m_iRet = ERRNO_MODEMVOLUME;
                else
                    m_Parent->setVolume( tone );
            }
            else {
                ret = m_modem->dtmf( tone );
                if  ( ret < 0 )
                    m_iRet = ERRNO_DTMF;
            }

            if  ( ret < 0 )
                return;  // stop this thing on error
        }
    }

    QQueue<char> m_Queue;
};

class ModemProcThread : public ModemOpThread {
public:
    ModemProcThread( PhoneApplet* parent ):ModemOpThread( parent ) {
        m_proc.setProcessChannelMode( QProcess::MergedChannels );
    }

    void setCmd( QString cmd )      { m_cmd = cmd; }

    bool abort() {
        m_proc.kill();
        return ModemOpThread::abort();
    }

    QString output() {
        QString txt = m_local;
        txt += m_proc.readAllStandardOutput();
        m_local = "";
        return txt;
    }

    void startProc( QString cmd ) {
        echo( cmd );
        m_proc.start( inShell( cmd ) );   // this MUST run in a shell for wildcards
    }

    bool startDetached( QString cmd ) {
        echo( cmd );
        return m_proc.startDetached( inShell( cmd ) );
    }

    int execute( QString cmd ) {
        echo( cmd );
        return m_proc.execute( inShell( cmd ) );
    }

    void faxWildcard( QString& name ) {
        int ix = name.indexOf( '.' );
        if  ( ix < 0 )
            name.append( '*');
        else
            name.insert( ix, '*' );
    }

    void echo( QString txt ) { m_local = txt; m_local += '\n'; }
    QString inShell( QString cmd ) { return "sh -c \"" + cmd + "\""; }


protected:
    QString m_cmd;
    QString m_local;
    QProcess m_proc;
};

class SendFaxThread : public ModemProcThread {
public:
    SendFaxThread( PhoneApplet* parent ):ModemProcThread( parent ) {
    }

    void run() {
        init();
        m_bConverting = FALSE;
        m_faxes = "";
        m_tempfs.clear();

        int nFiles = m_files.size();
        QTemporaryFile* temp;
        QString tempFilename;

        for ( int n = 0; !m_bAbort && n < nFiles; ++n ) {
            if  ( n > 0 )
                m_faxes += ' ';     // some will want ,
            m_bConverting = TRUE;
            KMimeType::Ptr mime = KMimeType::findByFileContent( m_files[ n ] );
            m_mimedesc = mime->comment();
            if  ( m_mimedesc.contains( "text") || m_mimedesc.contains( "g3" )) {
                // use it
                m_faxes += m_files[ n ];
            }
            else {
                // do the convertions
                temp = new QTemporaryFile( this );
                temp->setAutoRemove( FALSE );
                temp->open();
                tempFilename = temp->fileName();
                temp->close();  // so conversion can make use of it
                temp->remove(); // empty new one will be created

                m_exec = m_conv;
                if  ( m_conv.contains( CMDARG1 ))
                    m_exec.replace(CMDARG1, m_files[ n ]);
                else {
                    m_exec += " <";
                    m_exec += m_files[n];
                }
                if  ( m_conv.contains(CMDARG2 ))
                    m_exec.replace(CMDARG2, tempFilename);
                else {
                    m_exec += " >";
                    m_exec += tempFilename;
                }
                startProc( m_exec );
                m_proc.waitForFinished(-1);
                m_iRet = m_proc.exitCode();
                if  ( m_iRet != 0 ) {
                    m_iRet = ERRNO_FAXCONVERT;
                    return;
                }
                faxWildcard( tempFilename );      // support multipage -# before .
                m_faxes += tempFilename;
                m_tempfs << temp;       // for persistance
            }
        }
        m_bConverting = FALSE;
        if  ( m_bAbort )
            return;
        
        m_exec = m_cmd;
        if  ( m_exec.contains(CMDARG3))
            m_exec.replace( CMDARG3, m_faxes );
        else {
            m_exec += ' ';
            m_exec += m_faxes;
        }

        startProc( m_exec );
        m_proc.waitForFinished(-1);
        m_iRet = m_proc.exitCode();
        if  ( m_iRet == 1 )
            m_iRet = ERRNO_BUSY;
        else if ( m_iRet != 0 )
            m_iRet = ERRNO_SENDFAXERROR;
    }

    void setConv( QString cmd )   { m_conv = cmd; }
    void setFiles( QString files)   { m_files = files.split(" "); }
    bool converting()               { return m_bConverting; }

private:
    QStringList m_files;
    QString m_faxes;
    QList<QTemporaryFile*> m_tempfs;    // justkeep for persistance
    QString m_conv;
    QString m_mimedesc;
    QString m_exec;
    bool m_bConverting;
};

class RecvFaxThread : public ModemProcThread {
public:
    RecvFaxThread( PhoneApplet* parent ):ModemProcThread( parent ) {
    }

    void run() {
        QTemporaryFile* temp;
        QString tempFilename;

        init();

        temp = new QTemporaryFile(m_prefix, this);
        temp->setAutoRemove(FALSE);
        temp->open();
        tempFilename = temp->fileName();
        temp->close(); // so conversion can make use of it

        m_exec = m_cmd;
        if  ( m_exec.contains(CMDARG3))
            m_exec.replace( CMDARG3, tempFilename );
        else {
            m_exec += ' ';
            m_exec += tempFilename;
        }
        startProc(m_exec);
        m_proc.waitForFinished(-1);
        m_iRet = m_proc.exitCode();
        if ( m_iRet != 0 )
            m_iRet = ERRNO_RECVFAXERROR;

        faxWildcard( tempFilename );
        if  ( m_iRet == 0 && m_viewer.length() > 0 && !m_bAbort ) {
            m_exec = m_viewer;
            if  ( m_exec.contains( CMDARG1 ))
                m_exec.replace(CMDARG1, tempFilename );
            else {
                m_exec += ' ';
                m_exec += tempFilename;
            }

            startDetached( m_exec );
        }
    }

    void setPrefix( QString cmd )   { m_prefix = cmd; }
    void setViewer( QString cmd )   { m_viewer = cmd; }

private:
    QString m_prefix;
    QString m_viewer;
    QString m_exec;
};


#endif	/* _PHONEAPPLET_THREADS_H */

